/***************************************************************************
 *   copyright       : (C) 2006-2009 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "geometriedialog.h"
#include "aidedialog.h"
#include "logdialog.h"
#include "eukparser.h"
#include "objetdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QFont>
#include <QColor>
#include <QHeaderView>
#include <QUrl>
#include <QTextCodec>
#include <QFontDatabase>
#include <QProgressBar>

#include <math.h>

GeometrieDialog::GeometrieDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

completer = new QCompleter(this);
QAbstractItemModel *model;
QFile completefile(":/documents/completion.txt");
if (!completefile.open(QFile::ReadOnly)) model=new QStringListModel(completer);
QStringList words;
QString line;
QTextStream tscompleter(&completefile);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
tscompleter.setCodec(codec);
while (!tscompleter.atEnd()) 
	{
	line = tscompleter.readLine();
	if (!line.isEmpty()) words.append(line.remove("\n"));
	}
words.sort();
model=new QStringListModel(words, completer);
completer->setModel(model);
completer->setModelSorting(QCompleter::CaseInsensitivelySortedModel);
completer->setCaseSensitivity(Qt::CaseInsensitive);
completer->setWrapAround(false);
ui.EditorView->editor->setCompleter(completer);

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouvelleFigure()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerFigure()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverFigure()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.checkBoxAxes, SIGNAL(toggled(bool)),this, SLOT(ActiverAxes(bool)));
connect(ui.checkBoxGrille, SIGNAL(toggled(bool)),this, SLOT(ActiverGrille(bool)));

connect(ui.lineEditUniteX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditXmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradX, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.lineEditUniteY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmin, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditYmax, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGrilleY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));
connect(ui.lineEditGradY, SIGNAL(textChanged(const QString &)),this,SLOT(NettoyerCode()));

connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

connect(ui.EditorView->editor, SIGNAL(textChanged()), this, SLOT(NettoyerCode()));
ui.EditorView->editor->document()->setModified(false);

connect(ui.checkBoxRepere, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));
connect(ui.checkBoxLabels, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));



connect(ui.effacerButton, SIGNAL(clicked()), this, SLOT(NettoyerSupp()));
connect(ui.effacerInputButton, SIGNAL(clicked()), this, SLOT(NettoyerCommandes()));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";
eukTempFile=prefixFile+".euk";

highlighterSupp = new Highlighter(ui.textEditSupp->document());
QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);

ConstruireAideCommandes();
Init();

estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Géométrie : ")+nomFichier);
show();
ui.EditorView->editor->setFocus();
}

GeometrieDialog::~GeometrieDialog(){
if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running) 
	{
	asy_pdf_proc->disconnect();
	asy_pdf_proc->kill(); 
	delete asy_pdf_proc ;
	}
if (progressDialog) delete progressDialog;
}

void GeometrieDialog::ConstruireAideCommandes()
{
QStringList commandeList,commentaireList,codeList;
QFont fontCommande=qApp->font();
fontCommande.setBold(true);
QFont fontCommentaire=qApp->font();
//fontCommentaire.setItalic(true);
#if defined(Q_OS_MAC)
fontCommentaire.setPointSize(10);
#else
fontCommentaire.setPointSize(8);
#endif
QListWidgetItem *commande, *commentaire;

//dessin
dessinListWidget=new QListWidget(ui.toolBoxDessin);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "draw(A[,m])";
codeList << QString::fromUtf8("draw(•)");
commentaireList << QString::fromUtf8("Dessine le point A.\nOptions : m (dot, cross, plus)");

commandeList << "draw(u,A[,m])";
codeList << QString::fromUtf8("draw(•,•)");
commentaireList << QString::fromUtf8("Dessine un vecteur u à partir du point A \nOptions : m (full, dashed ,dotted)");

commandeList << "draw(d[,m[,m']])";
codeList << QString::fromUtf8("draw(•)");
commentaireList << QString::fromUtf8("Dessine une droite d.\nOptions : m (full, dashed ,dotted)\net m'(entire, halfline, backhalfline)");

commandeList << "draw(s[,m[,m']])";
codeList << QString::fromUtf8("draw(•)");
commentaireList << QString::fromUtf8("Dessine un segment s.\nOptions : m (full, dashed ,dotted)\net m'(entire, halfline, backhalfline)");

commandeList << "draw(c[,m])";
codeList << QString::fromUtf8("draw(•)");
commentaireList << QString::fromUtf8("Dessine un cercle c.\nOptions : m (full, dashed ,dotted)");

commandeList << "draw(c,a:,a':[,m[,m]])";
codeList << QString::fromUtf8("draw(•,•:,•:)");
commentaireList << QString::fromUtf8("Dessine un arc du cercle c\ndu point d'argument a au point d'argument a'.\nOptions : m (full, dashed ,dotted)\net m' (noarrow, arrow, backarrow , doublearrow)");

commandeList << "draw(A,B,C[,m])";
codeList << QString::fromUtf8("draw(•,•,•)");
commentaireList << QString::fromUtf8("Dessine un triangle ABC.\nOptions : m (full, dashed ,dotted)");

commandeList << "draw(A,B,C,D[,m])";
codeList << QString::fromUtf8("draw(•,•,•,•)");
commentaireList << QString::fromUtf8("Dessine un quadrilatère ABCD.\nOptions : m (full, dashed ,dotted)");

commandeList << "draw(A,B,C,D,E[,m])";
codeList << QString::fromUtf8("draw(•,•,•,•,•)");
commentaireList << QString::fromUtf8("Dessine un pentagone ABCDE.\nOptions : m (full, dashed ,dotted)");

commandeList << "draw(A,B,C,D,E,F[,m])";
codeList << QString::fromUtf8("draw(•,•,•,•,•,•)");
commentaireList << QString::fromUtf8("Dessine un hexagone ABCDEF.\nOptions : m (full, dashed ,dotted)");

commandeList << "draw(\"ch\",A,[x,]a:)";
codeList << QString::fromUtf8("draw(\"•\",•,•:)");
commentaireList << QString::fromUtf8("Affiche le texte contenu dans ch à une distance x\net avec un argument a par rapport au point A.");

commandeList << "draw(\"ch\",s,[x,]a:)";
codeList << QString::fromUtf8("draw(\"•\",•,•:)");
commentaireList << QString::fromUtf8("Affiche le texte contenu dans ch à une distance x\net avec un argument a par rapport au milieu du segment s.");

commandeList << "mark(s[,m])";
codeList << QString::fromUtf8("mark(•)");
commentaireList << QString::fromUtf8("Marque le segment s avec un symbole correspondant\nà m (simple, double, triple , cross)");

commandeList << "mark(A,B,C[,m])";
codeList << QString::fromUtf8("mark(•,•,•)");
commentaireList << QString::fromUtf8("Marque l'angle ABC avec un symbole correspondant\nà m (simple, double, triple, dashed, right, dot, arrow , backarrow)");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(dessinListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#invalide"));
commentaire=new QListWidgetItem(dessinListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBoxDessin->removeItem(0);
ui.toolBoxDessin->addItem(dessinListWidget,QString::fromUtf8("Dessin"));
connect(dessinListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//points
pointsListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "point(x,y)";
codeList << QString::fromUtf8("point(•,•)");
commentaireList << QString::fromUtf8("Point d'abscisse x et d'ordonnée y");

commandeList << "point(x,a:)";
codeList << QString::fromUtf8("point(•,•:)");
commentaireList << QString::fromUtf8("Point de coordonnées polaires x (rayon) et a (angle)");

commandeList << "point(c,a:)";
codeList << QString::fromUtf8("point(•,•:)");
commentaireList << QString::fromUtf8("Point d'argument a (en degrés) sur le cercle c");

commandeList << "barycenter(A[,x],B[,y])";
codeList << QString::fromUtf8("barycenter(•,•,•,•)");
commentaireList << QString::fromUtf8("Barycentre de A et B avec les coefficients x et y\n(même syntaxe pour 3 ou 4 points)");

commandeList << "intersection(d,d')";
codeList << QString::fromUtf8("intersection(•,•)");
commentaireList << QString::fromUtf8("Point d'intersection des droites d et d'");

commandeList << "midpoint(s)";
codeList << QString::fromUtf8("midpoint(•)");
commentaireList << QString::fromUtf8("Milieu du segment s");

commandeList << "orthocenter(A,B,C)";
codeList << QString::fromUtf8("orthocenter(•,•,•)");
commentaireList << QString::fromUtf8("Orthocentre du triangle ABC");

commandeList << "projection(A,d[,d'])";
codeList << QString::fromUtf8("projection(•,•)");
commentaireList << QString::fromUtf8("Projeté du point A sur la droite d suivant la direction de la droite d'\n(par défaut : perpendiculaire à la droite d)");

commandeList << "translation(A,u)";
codeList << QString::fromUtf8("translation(•,•)");
commentaireList << QString::fromUtf8("Image du point A par la translation de vecteur u");

commandeList << "reflection(A,d)";
codeList << QString::fromUtf8("reflection(•,•)");
commentaireList << QString::fromUtf8("Image du point A par la réflexion d'axe d");

commandeList << "rotation(A,B[,a:])";
codeList << QString::fromUtf8("rotation(•,•,•:)");
commentaireList << QString::fromUtf8("Image du point A par la rotation de centre B et\nd'angle a (par défaut : 180°)");

commandeList << "center(c)";
codeList << QString::fromUtf8("center(•)");
commentaireList << QString::fromUtf8("Centre du cercle c");

commandeList << "begin(s)";
codeList << QString::fromUtf8("begin(•)");
commentaireList << QString::fromUtf8("Origine du segment s");

commandeList << "end(s)";
codeList << QString::fromUtf8("end(•)");
commentaireList << QString::fromUtf8("Extrémité du segment s");

commandeList << "abscissa(d,x)";
codeList << QString::fromUtf8("abscissa(•,•)");
commentaireList << QString::fromUtf8("Point d'abscisse x sur la droite d");

commandeList << "ordinate(d,y)";
codeList << QString::fromUtf8("ordinate(•,•)");
commentaireList << QString::fromUtf8("Point d'ordonnée y sur la droite d.");

commandeList << "point(d,x)";
codeList << QString::fromUtf8("point(•,•)");
commentaireList << QString::fromUtf8("Point d'abscisse x sur la droite graduée d");

commandeList << "point(s,x)";
codeList << QString::fromUtf8("point(•,•)");
commentaireList << QString::fromUtf8("Point d'abscisse x sur la droite graduée contenant le segment s");


for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(pointsListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#point"));
commentaire=new QListWidgetItem(pointsListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(pointsListWidget,QString::fromUtf8("Points"));
connect(pointsListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//lignes
lignesListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "line(A,B)";
codeList << QString::fromUtf8("line(•,•)");
commentaireList << QString::fromUtf8("Droite passant par les points A et B");

commandeList << "line(A,u)";
codeList << QString::fromUtf8("line(•,•)");
commentaireList << QString::fromUtf8("Droite passant par le point A et de vecteur directeur u");

commandeList << "line(s)";
codeList << QString::fromUtf8("line(•)");
commentaireList << QString::fromUtf8("Droite contenant le segment s");

commandeList << "line(A,a:)";
codeList << QString::fromUtf8("line(•,•:)");
commentaireList << QString::fromUtf8("Droite passant par le point A et d'angle polaire a");

commandeList << "line(c,a:)";
codeList << QString::fromUtf8("line(•,•:)");
commentaireList << QString::fromUtf8("Tangente au cercle c au point d'argument a");

commandeList << "parallel(d,A)";
codeList << QString::fromUtf8("parallel(•,•)");
commentaireList << QString::fromUtf8("Droite passant par le point A et parallèle à la droite d");

commandeList << "parallel(s,A)";
codeList << QString::fromUtf8("parallel(•,•)");
commentaireList << QString::fromUtf8("Droite passant par le point A et parallèle au segment s");

commandeList << "perpendicular(d,A)";
codeList << QString::fromUtf8("perpendicular(•,•)");
commentaireList << QString::fromUtf8("Droite passant par le point A et perpendiculaire à la droite d");

commandeList << "perpendicular(s,A)";
codeList << QString::fromUtf8("perpendicular(•,•)");
commentaireList << QString::fromUtf8("Droite passant par le point A et perpendiculaire au segment s");

commandeList << "bisector(s)";
codeList << QString::fromUtf8("bisector(•)");
commentaireList << QString::fromUtf8("Médiatrice du segment s");

commandeList << "bisector(A,B,C)";
codeList << QString::fromUtf8("bisector(•,•,•)");
commentaireList << QString::fromUtf8("Bissectrice de l'angle ABC");

commandeList << "bisector(d,d')";
codeList << QString::fromUtf8("bisector(•,•)");
commentaireList << QString::fromUtf8("Bissectrice de l'angle aigu formé par d et d'");

commandeList << "altitude(A,B,C)";
codeList << QString::fromUtf8("altitude(•,•,•)");
commentaireList << QString::fromUtf8("Hauteur du triangle ABC passant par A");

commandeList << "median(A,B,C)";
codeList << QString::fromUtf8("median(•,•,•)");
commentaireList << QString::fromUtf8("Médiane du triangle ABC passant par A");

commandeList << "translation(d,u)";
codeList << QString::fromUtf8("translation(•,•)");
commentaireList << QString::fromUtf8("Image de la droite d par la translation d e vecteur u");

commandeList << "reflection(d,d')";
codeList << QString::fromUtf8("reflection(•,•)");
commentaireList << QString::fromUtf8("Image de la droite d par la réflexion d'axe d'");

commandeList << "rotation(d, A[,a:])";
codeList << QString::fromUtf8("rotation(•,•,•:)");
commentaireList << QString::fromUtf8("Image de la droite d par la rotation de centre A et d'angle a\n(par défaut : 180°)");

commandeList << "homothecy(d,A,x)";
codeList << QString::fromUtf8("homothecy(•,•,•)");
commentaireList << QString::fromUtf8("Image de la droite d par l'homothétie de centre A et de rapport x");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(lignesListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#droite"));
commentaire=new QListWidgetItem(lignesListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(lignesListWidget,QString::fromUtf8("Lignes"));
connect(lignesListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//segments
segmentsListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "segment(A,B)";
codeList << QString::fromUtf8("segment(•,•)");
commentaireList << QString::fromUtf8("Segment défini par les points A et B");

commandeList << "segment(A,u)";
codeList << QString::fromUtf8("segment(•,•)");
commentaireList << QString::fromUtf8("Segment défini par le point A et le vecteur u");

commandeList << "segment(A,x,a:)";
codeList << QString::fromUtf8("segment(•,•,•:)");
commentaireList << QString::fromUtf8("Segment d'origine A, de longueur x et d'angle polaire a");

commandeList << "segment(c,a:)";
codeList << QString::fromUtf8("segment(•,•:)");
commentaireList << QString::fromUtf8("Rayon du cercle c d'argument a");

commandeList << "translation(s,u)";
codeList << QString::fromUtf8("translation(•,•)");
commentaireList << QString::fromUtf8("Image du segment s par la translation de vecteur u");

commandeList << "reflection(s,d)";
codeList << QString::fromUtf8("reflection(•,•)");
commentaireList << QString::fromUtf8("Image du segment s par la réflexion d'axe d");

commandeList << "rotation(s,A[,a:])";
codeList << QString::fromUtf8("rotation(•,•,•:)");
commentaireList << QString::fromUtf8("Image du segment s par la rotation de centre A et d'angle a\n(par défaut : 180°)");

commandeList << "homothecy(s,A,x)";
codeList << QString::fromUtf8("homothecy(•,•,•)");
commentaireList << QString::fromUtf8("Image du segment s par l'homothétie de centre A et de rapport x");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(segmentsListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#segment"));
commentaire=new QListWidgetItem(segmentsListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(segmentsListWidget,QString::fromUtf8("Segments"));
connect(segmentsListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//cercles
cerclesListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "circle(A,B)";
codeList << QString::fromUtf8("circle(•,•)");
commentaireList << QString::fromUtf8("Cercle de diamètre [AB]");

commandeList << "circle(A,B,C)";
codeList << QString::fromUtf8("circle(•,•,•)");
commentaireList << QString::fromUtf8("Cercle circonscrit au triangle ABC");

commandeList << "circle(A,x)";
codeList << QString::fromUtf8("circle(•,•)");
commentaireList << QString::fromUtf8("Cercle de centre A et de rayon x");

commandeList << "incircle(A,B,C)";
codeList << QString::fromUtf8("incircle(•,•,•)");
commentaireList << QString::fromUtf8("Cercle inscrit dans le triangle ABC");

commandeList << "translation(c,u)";
codeList << QString::fromUtf8("translation(•,•)");
commentaireList << QString::fromUtf8("Image du cercle c par la translation de vecteur u");

commandeList << "reflection(c,d)";
codeList << QString::fromUtf8("reflection(•,•)");
commentaireList << QString::fromUtf8("Image du cercle c par la réflexion d'axe d");

commandeList << "rotation(c,A[,a:])";
codeList << QString::fromUtf8("rotation(•,•,•:)");
commentaireList << QString::fromUtf8("Image du cercle c par la rotation de centre A et d'angle a\n(par défaut : 180°)");

commandeList << "homothecy(c,A,x)";
codeList << QString::fromUtf8("homothecy(•,•,•)");
commentaireList << QString::fromUtf8("Image du cercle c par l'homothétie de centre A et de rapport x");

commandeList << "A B intersection(d,c)";
codeList << QString::fromUtf8("• • intersection(•,•)");
commentaireList << QString::fromUtf8("Définit les points A et B comme intersection de la droite d\net du cercle c");

commandeList << "A B intersection(c,c')";
codeList << QString::fromUtf8("• • intersection(•,•)");
commentaireList << QString::fromUtf8("Définit les points A et B comme intersection des cercles c et c'");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(cerclesListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
if (i<=7) commande->setData(Qt::UserRole,codeList.at(i)+QString("#cercle"));
else commande->setData(Qt::UserRole,codeList.at(i)+QString("#invalide"));
commentaire=new QListWidgetItem(cerclesListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(cerclesListWidget,QString::fromUtf8("Cercles"));
connect(cerclesListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//triangles
trianglesListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "A B C triangle[(x[,b:])]";
codeList << QString::fromUtf8("• • • triangle");
commentaireList << QString::fromUtf8("Définit un triangle ABC quelconque\nOptions : AB = x ; b=angle polaire du segment [AB]");

commandeList << "A B C triangle(x,y,z[,b:])";
codeList << QString::fromUtf8("• • • triangle(•,•,•)");
commentaireList << QString::fromUtf8("Définit un triangle ABC avec AB=x, BC=y et AC=z\nOption : b=angle polaire du segment [AB]");

commandeList << "A B C triangle(x,a,a'[,b:])]";
codeList << QString::fromUtf8("• • • triangle(•,•,•)");
commentaireList << QString::fromUtf8("Définit un triangle ABC avec AB=x, angle BAC=a et angle CBA=a'\nOption : b=angle polaire du segment [AB]");

commandeList << "A B C right[(x,y[,a:])]";
codeList << QString::fromUtf8("• • • right(•,•)");
commentaireList << QString::fromUtf8("Définit un triangle rectangle en A\nOptions : AB=x, AC=y, b=angle polaire du segment [AB]");

commandeList << "A B C right(x,a:[,b:])";
codeList << QString::fromUtf8("• • • triangle(•,•:)");
commentaireList << QString::fromUtf8("Définit un triangle rectangle en B tel que AB=x et angle BAC=a\nOption : b=angle polaire du segment [AB]");

commandeList << "A B C isosceles[(x,a:[,b:])]";
codeList << QString::fromUtf8("• • • isosceles(•,•:)");
commentaireList << QString::fromUtf8("Définit un triangle isocèle en C\nOptions : AB =x , angles BAC et CBA=a\nb=angle polaire du segment [AB]");

commandeList << "A B C isosceles(x,y[,b:])";
codeList << QString::fromUtf8("• • • isosceles(•,•)");
commentaireList << QString::fromUtf8("Définit un triangle isocèle en C avec AB=x  et AC=BC=y\nb=angle polaire du segment [AB]");

commandeList << "A B C equilateral[(x[,b:])]";
codeList << QString::fromUtf8("• • • equilateral(•)");
commentaireList << QString::fromUtf8("Définit un triangle équilatéral\nOptions : côté=x et b=angle polaire du segment [AB]");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(trianglesListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#invalide"));
commentaire=new QListWidgetItem(trianglesListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(trianglesListWidget,QString::fromUtf8("Triangles"));
connect(trianglesListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//Polygones
polygonesListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "A B C D parallelogram[(x,y,a:[,b:])]";
codeList << QString::fromUtf8("• • • • parallelogram(•,•,•:)");
commentaireList << QString::fromUtf8("Définit un parallélogramme\nOptions : AB=x , AD=y et angle BAD=a\nb=angle polaire du segment [AB]");

commandeList << "A B C D parallelogram(u,v[,b:])";
codeList << QString::fromUtf8("• • • • parallelogram(•,•)");
commentaireList << QString::fromUtf8("Définit un parallélogramme de vecteurs AB=u et AD=v\nOption : b=angle polaire du segment [AB]");

commandeList << "A B C D rectangle[(x,y[,b:])]";
codeList << QString::fromUtf8("• • • • rectangle(•,•)");
commentaireList << QString::fromUtf8("Définit un rectangle\nOptions : AB=x , AD=y et b=angle polaire du segment [AB]");

commandeList << "A B C D square[(x[,b:])]";
codeList << QString::fromUtf8("• • • • square(•)");
commentaireList << QString::fromUtf8("Définit un carré\nOptions : côté=x et b=angle polaire du segment [AB]");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(polygonesListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#invalide"));
commentaire=new QListWidgetItem(polygonesListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(polygonesListWidget,QString::fromUtf8("Polygones"));
connect(polygonesListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));

//Vecteurs
vecteursListWidget=new QListWidget(ui.toolBox);

commandeList.clear();
commentaireList.clear();
codeList.clear();

commandeList << "vector(x,y)";
codeList << QString::fromUtf8("vector(•,•)");
commentaireList << QString::fromUtf8("Définit un vecteur de coordonnées x et y");

commandeList << "vector(x,a:)";
codeList << QString::fromUtf8("vector(•,•:)");
commentaireList << QString::fromUtf8("Définit un vecteur de longueur x et d'argument a");

commandeList << "vector(A,B)";
codeList << QString::fromUtf8("vector(•,•)");
commentaireList << QString::fromUtf8("Définit un vecteur à partir des points A et B");

commandeList << "vector(l)";
codeList << QString::fromUtf8("vector(•)");
commentaireList << QString::fromUtf8("Définit un vecteur unitaire de même direction que la droite l");

commandeList << "vector(s)";
codeList << QString::fromUtf8("vector(•)");
commentaireList << QString::fromUtf8("Définit un vecteur à partir du segment s");

commandeList << "rotation(u,a:)";
codeList << QString::fromUtf8("rotation(•,•:)");
commentaireList << QString::fromUtf8("Image du vecteur u par la rotation d'angle a");

for (int i = 0; i < commandeList.count(); i++)
{
commande=new QListWidgetItem(vecteursListWidget);
commande->setFont(fontCommande);
commande->setText(commandeList.at(i));
commande->setData(Qt::UserRole,codeList.at(i)+QString("#vecteur"));
commentaire=new QListWidgetItem(vecteursListWidget);
commentaire->setFont(fontCommentaire);
commentaire->setText(commentaireList.at(i));
commentaire->setFlags(Qt::ItemIsEnabled);
}
ui.toolBox->addItem(vecteursListWidget,QString::fromUtf8("Vecteurs"));
connect(vecteursListWidget, SIGNAL(itemClicked ( QListWidgetItem*)), this, SLOT(InsererCommande(QListWidgetItem*)));
}

void GeometrieDialog::InsererCommande(QListWidgetItem* item)
{
if (item->font().bold())
	{
	QStringList data=item->data(Qt::UserRole).toString().split("#");
	QString tag=data.at(0);
	QString type=data.at(1);
	if ((type!="invalide") && (ui.EditorView->editor->textCursor().block().text().isEmpty()))
	    {
	    ObjetDialog *objetDialog = new ObjetDialog(this);
	    QString commande="";
	    if (type=="point")
	      {
	      objetDialog->ui.checkBoxNommer->setText(QString::fromUtf8("Nommer le point :"));
	      objetDialog->ui.checkBoxDessiner->setText(QString::fromUtf8("Tracer le point"));
	      objetDialog->ui.checkBoxAfficherNom->setChecked(true);
	      if (objetDialog->exec())
		  {
		  if ((objetDialog->ui.checkBoxNommer->isChecked()) && (!objetDialog->ui.lineEditNom->text().isEmpty()))
		      {
		      commande=objetDialog->ui.lineEditNom->text()+"="+tag+"\n";
		      if (objetDialog->ui.checkBoxDessiner->isChecked()) commande+="draw("+objetDialog->ui.lineEditNom->text()+")\n";
		      if (objetDialog->ui.checkBoxAfficherNom->isChecked()) commande+="draw(\""+objetDialog->ui.lineEditNom->text()+"\","+objetDialog->ui.lineEditNom->text()+",45:)\n";
		      ui.EditorView->editor->insertTag(commande,0,0);
		      }
		  else ui.EditorView->editor->insertTag(tag,0,0);
		  }
	      else ui.EditorView->editor->insertTag(tag,0,0);
	      }
	    else if (type=="droite")
	      {
	      objetDialog->ui.checkBoxNommer->setText(QString::fromUtf8("Nommer la droite :"));
	      objetDialog->ui.checkBoxDessiner->setText(QString::fromUtf8("Tracer la droite"));
	      objetDialog->ui.checkBoxAfficherNom->setChecked(false);
	      objetDialog->ui.checkBoxAfficherNom->hide();
	      if (objetDialog->exec())
		  {
		  if ((objetDialog->ui.checkBoxNommer->isChecked()) && (!objetDialog->ui.lineEditNom->text().isEmpty()))
		      {
		      commande=objetDialog->ui.lineEditNom->text()+"="+tag+"\n";
		      if (objetDialog->ui.checkBoxDessiner->isChecked()) commande+="draw("+objetDialog->ui.lineEditNom->text()+")\n";
		      ui.EditorView->editor->insertTag(commande,0,0);
		      }
		  else ui.EditorView->editor->insertTag(tag,0,0);
		  }
	      else ui.EditorView->editor->insertTag(tag,0,0);
	      }
	    else if (type=="segment")
	      {
	      objetDialog->ui.checkBoxNommer->setText(QString::fromUtf8("Nommer le segment :"));
	      objetDialog->ui.checkBoxDessiner->setText(QString::fromUtf8("Tracer le segment"));
	      objetDialog->ui.checkBoxAfficherNom->setChecked(false);
	      if (objetDialog->exec())
		  {
		  if ((objetDialog->ui.checkBoxNommer->isChecked()) && (!objetDialog->ui.lineEditNom->text().isEmpty()))
		      {
		      commande=objetDialog->ui.lineEditNom->text()+"="+tag+"\n";
		      if (objetDialog->ui.checkBoxDessiner->isChecked()) commande+="draw("+objetDialog->ui.lineEditNom->text()+")\n";
		      if (objetDialog->ui.checkBoxAfficherNom->isChecked()) commande+="draw(\""+objetDialog->ui.lineEditNom->text()+"\","+objetDialog->ui.lineEditNom->text()+",45:)\n";
		      ui.EditorView->editor->insertTag(commande,0,0);
		      }
		  else ui.EditorView->editor->insertTag(tag,0,0);
		  }
	      else ui.EditorView->editor->insertTag(tag,0,0);
	      }
	    else if (type=="cercle")
	      {
	      objetDialog->ui.checkBoxNommer->setText(QString::fromUtf8("Nommer le cercle :"));
	      objetDialog->ui.checkBoxDessiner->setText(QString::fromUtf8("Tracer le cercle"));
	      objetDialog->ui.checkBoxAfficherNom->setChecked(false);
	      objetDialog->ui.checkBoxAfficherNom->hide();
	      if (objetDialog->exec())
		  {
		  if ((objetDialog->ui.checkBoxNommer->isChecked()) && (!objetDialog->ui.lineEditNom->text().isEmpty()))
		      {
		      commande=objetDialog->ui.lineEditNom->text()+"="+tag+"\n";
		      if (objetDialog->ui.checkBoxDessiner->isChecked()) commande+="draw("+objetDialog->ui.lineEditNom->text()+")\n";
		      ui.EditorView->editor->insertTag(commande,0,0);
		      }
		  else ui.EditorView->editor->insertTag(tag,0,0);
		  }
	      else ui.EditorView->editor->insertTag(tag,0,0);
	      }
	    else if (type=="vecteur")
	      {
	      objetDialog->ui.checkBoxNommer->setText(QString::fromUtf8("Nommer le vecteur :"));
	      objetDialog->ui.checkBoxDessiner->setText(QString::fromUtf8("Tracer le vecteur"));
	      objetDialog->ui.checkBoxAfficherNom->setChecked(false);
	      objetDialog->ui.checkBoxAfficherNom->hide();
	      if (objetDialog->exec())
		  {
		  if ((objetDialog->ui.checkBoxNommer->isChecked()) && (!objetDialog->ui.lineEditNom->text().isEmpty()))
		      {
		      commande=objetDialog->ui.lineEditNom->text()+"="+tag+"\n";
		      if (objetDialog->ui.checkBoxDessiner->isChecked()) commande+="draw("+objetDialog->ui.lineEditNom->text()+QString::fromUtf8(",•)\n");
		      ui.EditorView->editor->insertTag(commande,0,0);
		      }
		  else ui.EditorView->editor->insertTag(tag,0,0);
		  }
	      else ui.EditorView->editor->insertTag(tag,0,0);
	      }
	    }
	else ui.EditorView->editor->insertTag(tag,0,0);
	ui.EditorView->editor->setFocus();
	}
}

void GeometrieDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Géométrie : "),
					QString::fromUtf8("La figure courante a été modifiée.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverFigure();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void GeometrieDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Géométrie : "),
					QString::fromUtf8("La figure courante a été modifiée.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverFigure();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void GeometrieDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Géométrie : "),
					QString::fromUtf8("La figure courante a été modifiée.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverFigure();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void GeometrieDialog::Init()
{
ui.textEditCode->clear();
ui.checkBoxAxes->setChecked(false);
ui.checkBoxLabels->setChecked(false);
ui.checkBoxRepere->setChecked(false);
ui.checkBoxGrille->setChecked(false);
ui.lineEditUniteX->setText("1");
ui.lineEditXmin->setText("-2");
ui.lineEditXmax->setText("8");
ui.lineEditGradX->setText("1");
ui.lineEditGrilleX->setText("1");
ui.lineEditUniteY->setText("1");
ui.lineEditYmin->setText("-2");
ui.lineEditYmax->setText("6");
ui.lineEditGradY->setText("1");
ui.lineEditGrilleY->setText("1");
ui.textEditSupp->clear();
ui.EditorView->editor->clear();
}
//*********************************************
void GeometrieDialog::ActiverAxes(bool etat)
{
ui.lineEditGradX->setEnabled(etat);
ui.lineEditGradY->setEnabled(etat);
ui.checkBoxLabels->setEnabled(etat);
NettoyerCode();
}
void GeometrieDialog::ActiverGrille(bool etat)
{
ui.lineEditGrilleX->setEnabled(etat);
ui.lineEditGrilleY->setEnabled(etat);
NettoyerCode();
}

//******************************************************************
bool GeometrieDialog::Operateur(QString mot)
{
bool result=false;
if ( (mot=="(") || (mot==")") || (mot=="-") || (mot=="+") || (mot=="*") || (mot=="/") || (mot=="^") ) result=true;
return result;
}

bool GeometrieDialog::Fonction(QString mot)
{
bool result=false;
if ( (mot=="cos") || (mot=="sin") || (mot=="sqrt") || (mot=="ln") || (mot=="log") || (mot=="exp") || (mot=="tan") || (mot=="abs") || (mot=="acos") || (mot=="asin") || (mot=="atan") ) result=true;
return result;
}

int GeometrieDialog::Priorite(QString mot)
{
int result=-1;
if (mot=="(") result=0;
else if (mot==")") result=1;
else if (mot=="-") result=2;
else if (mot=="+") result=2;
else if (mot=="*") result=3;
else if (mot=="/") result=3;
else if (mot=="^") result=5;
else if (Fonction(mot)) result=4;
return result;
}

QString GeometrieDialog::CalculOp(QString n,QString m,QString x,QString variable)
{
double calcul=0;
double p,q;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (m=="x") q=variable.toDouble(&ok);
else q=m.toDouble(&ok);
if (x=="+") calcul=q+p;
if (x=="-") calcul=q-p;
if (x=="*") calcul=q*p;
if (x=="/") 
	{
	if (p!=0) calcul=q/p;
	else existvaleur=false;
	}
if (x=="^") 
{
	if ((floor(abs(p))==(abs(p))) && (p>=0)) 
		{
		if (p==0) calcul=1;
		if (p==1) calcul=q;
		if (p>1) 
			{
			calcul=1;
			for (int i = 1; i <=p; i++) calcul=calcul*q;
			}
		}
	else 
		{
		if (q>1e-4) calcul=exp(p*log(abs(q)));
		else existvaleur=false;
		}
}
return QString::number(calcul,'g',12);
}

QString GeometrieDialog::CalculAp(QString n,QString x,QString variable)
{
double calcul=0;
double p;
bool ok;
if (n=="x") p=variable.toDouble(&ok);
else p=n.toDouble(&ok);
if (x=="cos")  calcul=cos(p);
if (x=="sin")  calcul=sin(p);
if (x=="abs")  calcul=abs(p);
if (x=="sqrt") 
   {
   if (p>=0)  calcul=sqrt(p);
   else existvaleur=false;
   }
if (x=="ln") 
   {
   if (p>1e-50)  calcul=log(p);
   else existvaleur=false;
   }
if (x=="exp") 
   {
   if ((p<80) && (p>-80))  calcul=exp(p);
   else existvaleur=false;
   }
if (x=="tan") 
   {
   if (cos(p)!=0) calcul=tan(p);
   else existvaleur=false;
   }
if (x=="acos") 
   {
   if ((p<=1) && (p>=-1)) calcul=acos(p);
   else existvaleur=false;
   }
if (x=="asin") 
   {
   if ((p<=1) && (p>=-1)) calcul=asin(p);
   else existvaleur=false;
   }
if (x=="atan") calcul=atan(p);
if (x=="log") 
   {
   if (p>1e-50)  calcul=log10(p);
   else existvaleur=false;
   }
return QString::number(calcul,'g',12);
}


QString GeometrieDialog::CalculerExpression(QString expression, QString variable)
{
QStringList Pp,Rp,Ep,Cp;
Pp.clear();
Rp.clear();
Ep.clear();
Cp.clear();
QString lettre,mot,n,m,z,result;
//**** PARSEUR et CREATION PILES
int i;
bool fini;
if (expression.isEmpty()) return result;
expression+="f";
i=0;
fini=false;
do
	{
	lettre=expression.mid(i,1);
	i+=1;
	if (lettre=="-")
		{
		if (Ep.isEmpty()) Ep.append("0");
		else if (Ep.last()=="(") Ep.append("0");
		}
	if (Operateur(lettre) || lettre=="x") Ep.append(lettre);
	else
		{
		mot="";
		while ((!Operateur(lettre)) && (lettre!="f")) 
			{
			mot+=lettre;
			lettre=expression.mid(i,1);
			i+=1;
			}
		if (mot=="pi") mot="3.141592653589793238";
		if (!mot.isEmpty()) Ep.append(mot);
		if (lettre=="f") 
			{
			fini=true;
			Ep.append("f");
			}
		else Ep.append(lettre);
		}
	}
while (!fini);
//******* POSTFIXE
i=0;
mot="";
Pp.append("(");
mot=Ep.at(i);
while (mot!="f") 
	{
	if ((!Operateur(mot)) && (!Fonction(mot))) Rp.append(mot);
	else
		{
		if (mot=="(") Pp.append(mot);
		else
			{
			while (Priorite(Pp.last())>=Priorite(mot)) 
				{
				Rp.append(Pp.last());
				Pp.removeLast();
				}
			if (mot==")")  Pp.removeLast();
			if (mot!=")")  Pp.append(mot);
			}
		}
	i++;
	mot=Ep.at(i);
	}
while (Pp.last()!="(" && !Pp.isEmpty())
      {
      Rp.append(Pp.last());
      Pp.removeLast();
      }
if (Rp.isEmpty()) Rp.append("0");
//******* CALCULER
existvaleur=true;
for (int ind=0;ind<Rp.count();ind++) 
	{
	mot=Rp.at(ind);
	if (Operateur(mot)) 
		{
		n=Cp.last();
		Cp.removeLast();
		m=Cp.last();
		Cp.removeLast();
		z=CalculOp(n,m,mot,variable);
		Cp.append(z);
		}
	else
	{
	if (Fonction(mot))
		{
		n=Cp.last();
		Cp.removeLast();
		z=CalculAp(n,mot,variable);
		Cp.append(z);
		}
	else 
		{
		Cp.append(mot);
		}
	}
	}
if (Cp.first()=="x") result=variable;
else result=Cp.first();
if (!existvaleur) result="Erreur";
return result;
}

QString GeometrieDialog::CalculerExpressionArrondi(QString expression, QString variable)
{
QString result;
double num_result, num_resultbis;
int approx_result;
bool ok;
result=CalculerExpression(expression,variable);
if (result!="Erreur") 
	{
	num_result=result.toDouble(&ok);
	if (ok) 
		{
		num_resultbis=floor (num_result*1e5);
		approx_result=(int) num_resultbis;
		num_result=(double) (approx_result/1e5);
		result=QString::number(num_result,'g',12);
		}
	}
return result;
}
//******************************************************************
QString GeometrieDialog::GenererCode()
{
bool ok;

QString s_unitX=CalculerExpressionArrondi(ui.lineEditUniteX->text(),"1");
double unitX;
if (s_unitX!="Erreur") unitX=s_unitX.toDouble(&ok);
else unitX=1;

QString s_unitY=CalculerExpressionArrondi(ui.lineEditUniteY->text(),"1");
double unitY;
if (s_unitY!="Erreur") unitY=s_unitY.toDouble(&ok);
else unitY=1;


QString s_Xmin=CalculerExpressionArrondi(ui.lineEditXmin->text(),"1");
double Xmin;
if (s_Xmin!="Erreur") Xmin=s_Xmin.toDouble(&ok);
else Xmin=-2;

QString s_Ymin=CalculerExpressionArrondi(ui.lineEditYmin->text(),"1");
double Ymin;
if (s_Ymin!="Erreur") Ymin=s_Ymin.toDouble(&ok);
else Ymin=-2;

QString s_Xmax=CalculerExpressionArrondi(ui.lineEditXmax->text(),"1");
double Xmax;
if (s_Xmax!="Erreur") Xmax=s_Xmax.toDouble(&ok);
else Xmax=8;

QString s_Ymax=CalculerExpressionArrondi(ui.lineEditYmax->text(),"1");
double Ymax;
if (s_Ymax!="Erreur") Ymax=s_Ymax.toDouble(&ok);
else Ymax=6;

QString s_gradX=CalculerExpressionArrondi(ui.lineEditGradX->text(),"1");
double gradX;
if (s_gradX!="Erreur") gradX=s_gradX.toDouble(&ok);
else gradX=1;

QString s_gradY=CalculerExpressionArrondi(ui.lineEditGradY->text(),"1");
double gradY;
if (s_gradY!="Erreur") gradY=s_gradY.toDouble(&ok);
else gradY=1;

QString s_grilleX=CalculerExpressionArrondi(ui.lineEditGrilleX->text(),"1");
double grilleX;
if (s_grilleX!="Erreur") grilleX=s_grilleX.toDouble(&ok);
else grilleX=1;
if (grilleX<=0) grilleX=1;

QString s_grilleY=CalculerExpressionArrondi(ui.lineEditGrilleY->text(),"1");
double grilleY;
if (s_grilleY!="Erreur") grilleY=s_grilleY.toDouble(&ok);
else grilleY=1;
if (grilleY<=0) grilleY=1;

double grilleXmin=Xmin/grilleX;
double grilleXmax=Xmax/grilleX;
double grilleYmin=Ymin/grilleY;
double grilleYmax=Ymax/grilleY;
double grilleUnitX=grilleX*unitX;
double grilleUnitY=grilleY*unitY;

QString s_Ox="0";
QString s_Oy="0";
if (Xmin*Xmax>0) s_Ox=s_Xmin;
if (Ymin*Ymax>0) s_Oy=s_Ymin;
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="import graph;\n";
texcode+="path[] crossdot=new path[] {scale(0.75mm)*(W--E), scale(0.75mm)*(N--S)};\n";
texcode+="void cross(picture pic=currentpicture, pair z,pen p=currentpen)\n";
texcode+="{\n";
texcode+="pic.add(new void(frame f, transform t) {\n";
texcode+="for (int i=0; i < crossdot.length; ++i) draw(f,shift(t*z)*transform(p)*crossdot[i],p);\n";
texcode+="},true);\n";
texcode+="}\n";
texcode+="unitsize("+s_unitX+"cm,"+s_unitY+"cm);\n";
texcode+="xlimits("+s_Xmin+","+s_Xmax+");\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+");\n";

if (ui.checkBoxGrille->isChecked())
  {
   texcode+="xaxis(BottomTop,xmin="+s_Xmin+",xmax="+s_Xmax+",Ticks(\"%\",extend=true,Step="+s_grilleX+"),p=linewidth(0.5pt)+gray(0.4));\n";
   texcode+="yaxis(LeftRight,ymin="+s_Ymin+",ymax="+s_Ymax+",Ticks(\"%\",extend=true,Step="+s_grilleY+"),p=linewidth(0.5pt)+gray(0.4));\n";
  }


EukParser *pars=new EukParser(ui.EditorView->editor->toPlainText(),tempDir+"/"+eukTempFile);
texcode+=pars->parseEukText();
delete pars;
texcode+="xlimits("+s_Xmin+","+s_Xmax+",Crop);\n";
texcode+="ylimits("+s_Ymin+","+s_Ymax+",Crop);\n";
QString ticksX="";
QString ticksY="";
QString arrow=",Arrow(2mm)";
if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0)) arrow="";
if (ui.checkBoxAxes->isChecked())
  {
  if (ui.checkBoxLabels->isChecked())
    {
    if (ui.checkBoxRepere->isChecked())
	{
	ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradX+",Size=1mm)";
	ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradY+",Size=1mm)";
	}
    else
	{
	if (Ymin==0) ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradX+",Size=1mm)";
	else ticksX=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradX+",Size=1mm)";
	if (Xmin==0) ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,Step="+s_gradY+",Size=1mm)";	else ticksY=",Ticks(scale(.7)*Label(),beginlabel=true,endlabel=true,begin=true,end=true,NoZero,Step="+s_gradY+",Size=1mm)";	 	
	}
    }
  else
    {
    ticksX=",Ticks(\"%\",NoZero,Step="+s_gradX+",Size=1mm)";
    ticksY=",Ticks(\"%\",NoZero,Step="+s_gradY+",Size=1mm)";
    }
    texcode+="xaxis(axis=YEquals("+s_Oy+"),xmin="+s_Xmin+",xmax="+s_Xmax+ticksX+",p=linewidth(1pt)+black"+arrow+",true);\n";
    texcode+="yaxis(axis=XEquals("+s_Ox+"),ymin="+s_Ymin+",ymax="+s_Ymax+ticksY+",p=linewidth(1pt)+black"+arrow+",true);\n";
  }

if (ui.checkBoxRepere->isChecked() && (Xmin<=0) && (Xmax>=0) && (Ymin<=0) && (Ymax>=0))
  {
  texcode+="labelx(Label(\"$O$\",NoFill), 0, SW);\n";
  texcode+="draw(Label(\"$\\vec{\\imath}$\",NoFill), (0,0)--(1,0),S,Arrow(2mm));\n";
  texcode+="draw(Label(\"$\\vec{\\jmath}$\",NoFill), (0,0)--(0,1),W,Arrow(2mm));\n";
  texcode+="dot((0,0));\n";
  }
texcode+=QString::fromUtf8("// code supplémentaire\n")+ui.textEditSupp->toPlainText()+"\n"; 
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";

return texcode;
}

void GeometrieDialog::AfficherCode()
{
QString texcode="";
ui.textEditCode->clear();
texcode+=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//******************************************************
void GeometrieDialog::NouvelleFigure()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Géométrie : "),
					QString::fromUtf8("La figure courante a été modifiée.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverFigure();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Géométrie : ")+nomFichier);
}

void GeometrieDialog::SauverFigure()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Geometrie");
doc.appendChild(root);

description=doc.createElement("UnitX");
contenu=doc.createTextNode(ui.lineEditUniteX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmin");
contenu=doc.createTextNode(ui.lineEditXmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Xmax");
contenu=doc.createTextNode(ui.lineEditXmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("UnitY");
contenu=doc.createTextNode(ui.lineEditUniteY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymin");
contenu=doc.createTextNode(ui.lineEditYmin->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Ymax");
contenu=doc.createTextNode(ui.lineEditYmax->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradX");
contenu=doc.createTextNode(ui.lineEditGradX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GradY");
contenu=doc.createTextNode(ui.lineEditGradY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleX");
contenu=doc.createTextNode(ui.lineEditGrilleX->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("GrilleY");
contenu=doc.createTextNode(ui.lineEditGrilleY->text());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Axes");
if (ui.checkBoxAxes->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Labels");
if (ui.checkBoxLabels->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Repere");
if (ui.checkBoxRepere->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Grille");
if (ui.checkBoxGrille->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Commandes");
contenu=doc.createTextNode(ui.EditorView->editor->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Figure (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Figure (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Géométrie : ")+nomFichier);
}

void GeometrieDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Geometrie")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à une figure de géométrie"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("UnitX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Xmax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditXmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("UnitY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditUniteY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymin");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmin->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Ymax");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditYmax->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GradY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGradY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleX");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleX->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("GrilleY");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.lineEditGrilleY->setText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Axes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAxes->setChecked(true);
	else ui.checkBoxAxes->setChecked(false);
	}

nodelist = doc.elementsByTagName("Labels");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxLabels->setChecked(true);
	else ui.checkBoxLabels->setChecked(false);
	}

nodelist = doc.elementsByTagName("Repere");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxRepere->setChecked(true);
	else ui.checkBoxRepere->setChecked(false);
	}

nodelist = doc.elementsByTagName("Grille");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxGrille->setChecked(true);
	else ui.checkBoxGrille->setChecked(false);
	}

nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

nodelist = doc.elementsByTagName("Commandes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.EditorView->editor->setPlainText(node.toElement().text());
	}

fichier.close();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Géométrie : ")+nomFichier);
}

void GeometrieDialog::ChargerFigure()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Géométrie : "),
					QString::fromUtf8("La figure courante a été modifiée.\nVoulez-vous l'enregistrer avant de charger une autre figure?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverFigure();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Figure (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//******************************
void GeometrieDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Géométrie ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

void GeometrieDialog::NettoyerSupp()
{
ui.textEditSupp->selectAll();
ui.textEditSupp->cut();
NettoyerCode();
}

void GeometrieDialog::NettoyerCommandes()
{
ui.EditorView->editor->selectAll();
ui.EditorView->editor->cut();
NettoyerCode();
}

//*******************************
void GeometrieDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"geometrie");
aideDialog->exec();
}
//*****************************
void GeometrieDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void GeometrieDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void GeometrieDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void GeometrieDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
out.setCodec(codec);
out << GenererCode();
fichier.close();
erreurProcessus=false;
asy_pdf_proc = new QProcess( this );
asy_pdf_proc->setWorkingDirectory(tempDir);
QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
#if defined(Q_OS_MAC)
env.insert("PATH", env.value("PATH") + ":/Library/TeX/texbin:/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin:/usr/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/texbin:/sw/bin:");
asy_pdf_proc->setProcessEnvironment(env);
#endif
#if defined(Q_OS_WIN32)
env.insert("PATH", env.value("PATH") + ";c:/texlive/2018/bin;c:/texlive/2017/bin;c:/texlive/2016/bin;c:/texlive/2015/bin;c:/texlive/2014/bin;c:/texlive/2013/bin;c:/texlive/2012/bin;c:/texlive/2011/bin;c:/texlive/2010/bin;c:/w32tex/bin;c:/Program Files/MiKTeX 3.0/miktex/bin;c:/Program Files (x86)/MiKTeX 3.0/miktex/bin;c:/Program Files/MiKTeX 2.9/miktex/bin;c:/Program Files (x86)/MiKTeX 2.9/miktex/bin;c:/Program Files/MiKTeX 2.8/miktex/bin;c:/Program Files (x86)/MiKTeX 2.8/miktex/bin");
#endif
qputenv("PATH", env.value("PATH").toLatin1());

connect(asy_pdf_proc, SIGNAL(finished(int)),this, SLOT(finAsyPdfProcess(int)));
asyMessage="";
connect(asy_pdf_proc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
asy_pdf_proc->start(asyCommande+" -f pdf -noView "+asyTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(this);
QProgressBar* progress = new QProgressBar();
progress->setFormat(" ");
progress->setMaximum(0);
progress->setMinimum(0);
progressDialog->setBar(progress);
progressDialog->setLabelText(QString::fromUtf8("Compilation en cours..."));
progressDialog->setCancelButton(0);
progressDialog->setRange(0,0);
progressDialog->setMinimumDuration(0);
progressDialog->show();
progressDialog->raise();
}

void GeometrieDialog::readInfoOutput()
{
QByteArray result=asy_pdf_proc->readAllStandardError();
asyMessage=QString(result);
asyMessage=asyMessage.simplified();
}

void GeometrieDialog::finAsyPdfProcess(int err)
{
if (progressDialog) progressDialog->cancel();
if (err) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation Asymptote")+"\n"+asyMessage);
	erreurProcessus=true;
	}
else 
	{
	emit asy_pdf_fini();
	}
}


void GeometrieDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

