/***************************************************************************
 *   copyright       : (C) 2006-2012 by Pascal Brachet                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "graphedialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QTableView>
#include <QTableWidgetItem>
#include <QUrl>
#include <QFontDatabase>
#include <QProgressBar>


GrapheDialog::GrapheDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouveauGraphe()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerGraphe()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverGraphe()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.listWidgetSommets, SIGNAL(itemSelectionChanged()),this, SLOT(actualiserItemInfo()));
connect(ui.lineEditSommet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserSommet()));
connect(ui.lineEditAretes, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserAretes()));
connect(ui.ajouterButton, SIGNAL(clicked()), this, SLOT(ajouterSommet()));
connect(ui.supprimerButton, SIGNAL(clicked()), this, SLOT(supprimerSommet()));
connect(ui.checkBoxOrientation, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));

connect(ui.tableWidget,SIGNAL(cellChanged(int,int)),this, SLOT(NettoyerCode()));
connect( ui.spinBoxLignes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserLignes(int)));
connect( ui.spinBoxColonnes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserColonnes(int)));



connect(ui.matriceButton, SIGNAL(clicked()), this, SLOT(Matrice()));
connect(ui.colorationButton, SIGNAL(clicked()), this, SLOT(Coloration()));
connect(ui.chaineButton, SIGNAL(clicked()), this, SLOT(PlusCourteChaine()));

ui.listWidgetSommets->setAlternatingRowColors(true);
ui.listWidgetSommets->setSelectionMode (QAbstractItemView::SingleSelection);

ui.tableWidget->verticalHeader()->hide();
ui.tableWidget->horizontalHeader()->hide();

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";

highlighterSupp= new LatexHighlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditSupp->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);
ui.lineEditAretes->setFont(efont);
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Graphes : ")+nomFichier);
show();
}

GrapheDialog::~GrapheDialog(){
if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running) 
	{
	asy_pdf_proc->disconnect();
	asy_pdf_proc->kill(); 
	delete asy_pdf_proc ;
	}
if (progressDialog) delete progressDialog;
}

void GrapheDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Graphes : ",
					QString::fromUtf8("Le graphe courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverGraphe();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void GrapheDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Graphes : ",
					QString::fromUtf8("Le graphe courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverGraphe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void GrapheDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Graphes : ",
					QString::fromUtf8("Le graphe courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverGraphe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void GrapheDialog::Init()
{
ui.tableWidget->clear();
ui.tableWidget->setAcceptDrops(true);
ui.tableWidget->setDragEnabled(false);
ui.spinBoxLignes->setValue(4);
ActualiserLignes(4);
ui.spinBoxColonnes->setValue(8);
ActualiserColonnes(8);
ui.textEditCode->clear();
sommetList.clear();
aretesList.clear();
ui.listWidgetSommets->clear();
ui.listWidgetSommets->setDragEnabled(true);
ui.lineEditSommet->clear();
ui.lineEditAretes->clear();
ui.textEditSupp->clear();
ActiverSommets(false);
}

void GrapheDialog::ActualiserLignes(int num)
{
ui.tableWidget->setRowCount( num);
for ( int i = 0; i <num; ++i )
	{
	ui.tableWidget->setRowHeight(i,25);
	}
InitTableau();
}

void GrapheDialog::ActualiserColonnes(int num)
{
ui.tableWidget->setColumnCount( num );
for ( int i = 0; i <num; ++i )
	{
	ui.tableWidget->setColumnWidth(i,50);
	}
InitTableau();
}

void GrapheDialog::InitTableau()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item, *nouveau_item;
QString contenu;
for ( int li=0;li<nblignes;li++) 
	{
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li, col,nouveau_item);
			}
		}
	}
}

//*********************************************
void GrapheDialog::ActiverSommets(bool etat)
{
ui.lineEditSommet->setEnabled(etat);
ui.lineEditAretes->setEnabled(etat);
}
//******************************************************************
void GrapheDialog::GenererMatrice()
{
estPondere=false;
matrice.clear();
angleList.clear();
QStringList ligne, extremiteList;
QString extremite,poids,angle;
int col;
int nbsommets=sommetList.count();

for (int i = 0; i < nbsommets; i++)
	{
	ligne.clear();
	for (int j = 0; j < nbsommets; j++) ligne.append("pasadjacent");
	matrice.append(ligne);
	angleList.append("0");
	}

for (int li = 0; li < nbsommets; li++)
	{
	extremiteList.clear();
	if (!aretesList.at(li).isEmpty() && aretesList.at(li)!="0")
		{
		extremiteList=aretesList.at(li).split(":");
		for (int j = 0; j < extremiteList.count(); j++)
			{
			angle="";
			if (extremiteList.at(j).contains(";"))
				{
				extremite=extremiteList.at(j).section(";", 0, 0);
				poids=extremiteList.at(j).section(";", 1, 1);
				angle=extremiteList.at(j).section(";", 2, 2);
				if (poids!="0") estPondere=true;
				else poids="adjacent";
				}
			else
				{
				extremite=extremiteList.at(j);
				poids="adjacent";
				}
			col=sommetList.indexOf(extremite);
			if (col>-1)
				{
				ligne=matrice.at(li);
				ligne.replace(col,poids);
				matrice.replace(li,ligne);
				if (!angle.isEmpty()) angleList.replace(col,angle);
				if (!ui.checkBoxOrientation->isChecked()) 
					{
					ligne=matrice.at(col);
					ligne.replace(li,poids);
					matrice.replace(col,ligne);
					}
				}
			}
		}
	}
}

void GrapheDialog::Matrice()
{
GenererMatrice();
int nbsommets=sommetList.count();
QStringList ligne;
ui.textEditSupp->clear();
QString texcode="Matrice d'adjacence : \n";
texcode+="\\[\\left( \\begin{array}{";
QString item;
for (int i = 0; i < nbsommets; i++) texcode+="c";
texcode+="}\n";
for ( int i=0;i<nbsommets-1;i++) 
	{
	ligne=matrice.at(i);
	for ( int j=0;j<nbsommets-1;j++) 
		{
		item =ligne.at(j);
		if (item=="pasadjacent") texcode+="0 & ";
		else texcode+="1 & ";
		}
	item =ligne.at(nbsommets-1);
	if (item=="pasadjacent") texcode+="0  \\\\ \n";
	else texcode+="1  \\\\ \n";
	}
ligne=matrice.at(nbsommets-1);
for ( int j=0;j<nbsommets-1;j++) 
	{
	item =ligne.at(j);
	if (item=="pasadjacent") texcode+="0 & ";
	else texcode+="1 & ";
	}
item =ligne.at(nbsommets-1);
if (item=="pasadjacent") texcode+="0";
else texcode+="1";
texcode+="\n\\end{array} \\right) \\]";
ui.textEditSupp->setPlainText(texcode);
}

void GrapheDialog::Coloration()
{
GenererMatrice();
ui.textEditSupp->clear();
QStringList ligne;
QList<int> list_degre, list_couleur, list_memecouleur, position_matrice;
QStringList list_sommet=sommetList;
int nbsommets=list_sommet.count();
int d;
//initialisation
for (int i = 0; i < nbsommets; i++)
	{
	ligne=matrice.at(i);
	d=0;
	for (int j = 0; j < nbsommets; j++) 
		{
		if (ligne.at(j)!="pasadjacent") d+=1;
		}
	list_degre.append(d);
	list_couleur.append(0);
	}
//classement des sommets par ordre décroissant de leurs degrés
int max;
QString sommet;
for (int i = 0; i < nbsommets; i++)
	{
	max=i;
	for (int j = i+1; j < nbsommets; j++) 
		{
		if (list_degre.at(j)>list_degre.at(max)) max=j;
		}
	if (max!=i) 
		{
		list_degre.swap(i,max);
		list_sommet.swap(i,max);
		}
	}
//classement alphabétique pour les sommets de même degré
int min;
for (int i = 0; i < nbsommets; i++)
	{
	min=i;
	for (int j = i+1; j < nbsommets; j++) 
		{
		if (list_degre.at(j)==list_degre.at(min) && list_sommet.at(j)<list_sommet.at(min)) min=j;
		}
	if (min!=i) 
		{
		list_degre.swap(i,min);
		list_sommet.swap(i,min);
		}
	sommet=list_sommet.at(i);
	position_matrice.append(sommetList.indexOf(sommet));
	}

QString texcode=QString::fromUtf8("Liste des sommets ordonnés selon l'ordre décroissant de leur degré :\n");
texcode+="\\begin{center}\n\\begin{tabular}{|c|";
for ( int j=0;j<nbsommets;j++) texcode+="c|";
texcode+="}\n";
texcode+=QString::fromUtf8("\\hline Sommet & ");
for ( int j=0;j<nbsommets-1;j++) texcode+=list_sommet.at(j)+" & ";
texcode+=list_sommet.at(nbsommets-1)+" \\\\ \n";
texcode+=QString::fromUtf8("\\hline Degré & ");
for ( int j=0;j<nbsommets-1;j++) texcode+=QString::number(list_degre.at(j))+" & ";
texcode+=QString::number(list_degre.at(nbsommets-1))+" \\\\ \n";
texcode+="\\hline \n\\end{tabular}\n\\end{center}\n\n";
texcode+="\\begin{itemize}\n";
//coloration
bool continuer, go, adjacent;
int couleur=0;
int base,nouveau;
continuer=true;
while (continuer)
	{
	base=-1;
	for (int i = 0; (i < nbsommets) && (base==-1); i++)
		{
		if (list_couleur.at(i)==0) base=i;
		}
	if (base!=-1)
		{
		couleur+=1;
		texcode+=QString::fromUtf8("\\item On passe à la couleur ")+QString::number(couleur);
		list_couleur.replace(base,couleur);
		texcode+=QString::fromUtf8(" que l'on attribue au sommet ")+list_sommet.at(base)+" . ";
		go=true;
		while (go)
			{
			list_memecouleur.clear();
			for (int j = 0; j < nbsommets; j++)
				{
				if (list_couleur.at(j)==couleur) list_memecouleur.append(j);
				}
			nouveau=-1;
			for (int j = 0; (j<nbsommets) && (nouveau==-1); j++)
				{
				if (list_couleur.at(j)==0)
					{
					adjacent=false;
					for (int k = 0; (k<list_memecouleur.count()) && (!adjacent); k++)
						{
						ligne=matrice.at(position_matrice.at(j));
						if (ligne.at(position_matrice.at(list_memecouleur.at(k)))!="pasadjacent") adjacent=true;
						}
					if (!adjacent)
							{
							nouveau=j;
							texcode+=QString::fromUtf8("Le sommet ")+list_sommet.at(nouveau)+QString::fromUtf8(" n'est relié à aucun sommet de couleur ")+QString::number(couleur)+QString::fromUtf8(", on lui affecte donc cette couleur . ");
							list_couleur.replace(nouveau,couleur);
							}
					}
				}
			if (nouveau==-1) 
				{
				texcode+="\n";
				go=false;
				}
			}
		}
	else continuer=false;
	}
texcode+="\\end{itemize}\n\n";
texcode+=QString::fromUtf8("Résultat de la coloration :\n");
texcode+="\\begin{center}\n\\begin{tabular}{|c|";
for ( int j=0;j<nbsommets;j++) texcode+="c|";
texcode+="}\n";
texcode+=QString::fromUtf8("\\hline Sommet & ");
for ( int j=0;j<nbsommets-1;j++) texcode+=list_sommet.at(j)+" & ";
texcode+=list_sommet.at(nbsommets-1)+" \\\\ \n";
texcode+=QString::fromUtf8("\\hline Degré & ");
for ( int j=0;j<nbsommets-1;j++) texcode+=QString::number(list_degre.at(j))+" & ";
texcode+=QString::number(list_degre.at(nbsommets-1))+" \\\\ \n";
texcode+=QString::fromUtf8("\\hline Couleur & ");
for ( int j=0;j<nbsommets-1;j++) texcode+=QString::number(list_couleur.at(j))+" & ";
texcode+=QString::number(list_couleur.at(nbsommets-1))+" \\\\ \n";
texcode+="\\hline \n\\end{tabular}\n\\end{center}";
ui.textEditSupp->setPlainText(texcode);
}

void GrapheDialog::PlusCourteChaine()
{
GenererMatrice();
ui.textEditSupp->clear();
if (!estPondere) return;
QList<int> selection, pas_selection, origine;
QStringList coefficient, ligne;
int nbsommets=sommetList.count();
bool ok, erreur;
float coeff, coeff_courant, nouveau_coeff, ancien_coeff, coeff_min;

erreur=false;
for (int i = 0; i < nbsommets; i++)
	{
	ligne=matrice.at(i);
	for (int j = 0; j < nbsommets; j++) 
		{
		if (ligne.at(j)!="pasadjacent")
			{
			if (ligne.at(j)=="adjacent") erreur=true;
			else
				{
				coeff=ligne.at(j).toFloat(&ok);
				if (!ok) erreur=true;
				if (ok && coeff<0) erreur=true;
				}
			}
		}
	}
if (erreur) return;

int sommet_depart=numeroItem();
if (sommet_depart<0) return;
for (int i = 0; i < nbsommets; i++)
	{
	coefficient.append("infini");
	origine.append(-1);
	}
coefficient.replace(sommet_depart,"0");
selection.append(sommet_depart);
int sommet_courant=sommet_depart;
int etape=1;
bool continuer=true;
int min;
QString texcode=QString::fromUtf8("\\textit{(seuls les sommets dont le statut change sont indiqués)}\n");
texcode+="\\begin{itemize}\n";
texcode+=QString::fromUtf8("\\item \\'{E}tape ")+QString::number(etape)+QString::fromUtf8(" : sélection de ")+sommetList.at(sommet_depart)+"\n";
QString tag;
while (continuer)
	{
	etape+=1;
	tag="";
	ligne=matrice.at(sommet_courant);
	coeff_courant=coefficient.at(sommet_courant).toFloat(&ok);
	for (int j = 0; j < nbsommets ; j++)
		{
		if (!selection.contains(j) && ligne.at(j)!="pasadjacent")
			{
			coeff=ligne.at(j).toFloat(&ok);
			nouveau_coeff=coeff_courant+coeff;
			if (coefficient.at(j)=="infini") 
				{
				coefficient.replace(j,QString::number(nouveau_coeff));
				origine.replace(j,sommet_courant);
				tag+=sommetList.at(j)+":"+coefficient.at(j)+" de "+sommetList.at(sommet_courant)+" ; ";
				}
			else 
				{
				ancien_coeff=coefficient.at(j).toFloat(&ok);
				if (nouveau_coeff<ancien_coeff) 
					{
					coefficient.replace(j,QString::number(nouveau_coeff));
					origine.replace(j,sommet_courant);
					tag+=sommetList.at(j)+":"+coefficient.at(j)+" de "+sommetList.at(sommet_courant)+" ; ";
					}
				}
			}
		}
	pas_selection.clear();
	for (int j = 0; j < nbsommets ; j++)
		{
		if (!selection.contains(j) && coefficient.at(j)!="infini") pas_selection.append(j);
		}
	if (pas_selection.count()>0)
		{
		min=0;
		for (int k = 1; k < pas_selection.count() ; k++)
			{
			coeff_min=coefficient.at(pas_selection.at(min)).toFloat(&ok);
			coeff=coefficient.at(pas_selection.at(k)).toFloat(&ok);
			if (coeff<coeff_min) min=k;
			}
		sommet_courant=pas_selection.at(min);
		selection.append(sommet_courant);
		tag+=QString::fromUtf8("sélection de ")+sommetList.at(sommet_courant)+"\n";
		texcode+=QString::fromUtf8("\\item \\'{E}tape ")+QString::number(etape)+" : "+tag;
		}
	else 
		{
		continuer=false;
		}
	}
texcode+="\\end{itemize}\n\\medskip\n";
texcode+="\\begin{itemize}\n";
int precedent;
for (int i = 0; i < nbsommets; i++)
	{
	if (i!=sommet_depart)
		{
		tag=sommetList.at(i);
		precedent=origine.at(i);
		while (precedent!=-1)
			{
			tag=sommetList.at(precedent)+"-"+tag;
			precedent=origine.at(precedent);
			}
		texcode+=QString::fromUtf8("\\item Plus courte chaîne entre ")+sommetList.at(sommet_depart)+" et "+sommetList.at(i)+" : "+tag+" \n";
		}
	}
texcode+="\\end{itemize}";
ui.textEditSupp->setPlainText(texcode);
}
//******************************************************************
QString GrapheDialog::GenererCode()
{
GenererMatrice();
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
texcode+="unitsize(1cm,1cm);\n";
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QMap<QString, int> map_x;
QMap<QString, int> map_y;
QList<int> maxcol;
maxcol.clear();
int maxli,c;
maxli=0;
int vraili=0;
for ( int li=0;li<nblignes;li++) 
	{
	c=-1;
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li,col);
		if (item && !item->text().isEmpty()) 
			{
			c=col;
			map_x.insert(item->text(),2*col);
			map_y.insert(item->text(),-2*vraili);
			}
		}
	maxcol.append(c);
	if (c>=0) 
		{
		maxli=li;
		vraili+=1;
		}
	}

QStringList ligne,tligne;
QString origine,extremite,poids,tpoids;
int nbsommets=sommetList.count();
int origine_x,origine_y,extremite_x,extremite_y,dir_x,dir_y;
for (int li = 0; li < nbsommets; li++)
    {
    ligne=matrice.at(li);
    origine=sommetList.at(li);
    origine_x=map_x.value(origine);
    origine_y=map_y.value(origine);
    texcode+="object sommet"+QString::number(li)+"=draw(Label(\""+origine+"\"),ellipse,("+QString::number(origine_x)+","+QString::number(origine_y)+"),NoFill);\n";
    }
texcode+="add(new void(picture pic, transform t) {\n";
if (ui.checkBoxOrientation->isChecked())
{
for (int li = 0; li < nbsommets; li++)
  {
  ligne=matrice.at(li);
  origine=sommetList.at(li);
  origine_x=map_x.value(origine);
  origine_y=map_y.value(origine);
  poids=ligne.at(li);
  if (poids=="adjacent") 
  {
  if (angleList.at(li)=="-")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(180),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(-90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
//    texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{dir(180)}point(sommet"+QString::number(li)+",dir(0),t);\n";
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    }
  else if (angleList.at(li)=="+")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(180),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
    //texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{-dir(180)}point(sommet"+QString::number(li)+",dir(180),t);\n";
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    }
  else if (angleList.at(li)=="h")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(45),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(135),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
    //texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{-dir(180)}point(sommet"+QString::number(li)+",dir(180),t);\n"; 
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    }
  else if (angleList.at(li)=="b")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(-45),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(-135),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(-90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
    //texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{-dir(180)}point(sommet"+QString::number(li)+",dir(180),t);\n";
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    }
  //texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
  }
  else if (poids!="pasadjacent") 
  {
  if (angleList.at(li)=="-")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(0),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(-90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
//     texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{dir(180)}point(sommet"+QString::number(li)+",dir(0),t);\n";
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(li)+",.5),2*Relative(dir(90)),Fill(1,white));\n";
    }
  else if (angleList.at(li)=="+")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(180),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
//    texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(90),t){dir(90)}..{-dir(180)}point(sommet"+QString::number(li)+",dir(180),t);\n"; 
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(li)+",.5),2*Relative(dir(90)),Fill(1,white));\n";
    }
  else if (angleList.at(li)=="h")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(45),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(135),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
//    texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(135),t){dir(135)}..{dir(215)}point(sommet"+QString::number(li)+",dir(45),t);\n"; 
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(li)+",.5),Relative(dir(90)),Fill(1,white));\n";
    }
  else if (angleList.at(li)=="b")
    {
    texcode+="pair debut"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(-45),t);\n";
    texcode+="pair fin"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",dir(-135),t);\n";
    texcode+="pair mil"+QString::number(li)+QString::number(li)+"=(debut"+QString::number(li)+QString::number(li)+"+fin"+QString::number(li)+QString::number(li)+")/2;\n";
    texcode+="pair milieu"+QString::number(li)+QString::number(li)+"=shift(mil"+QString::number(li)+QString::number(li)+")*scale(2)*shift(-mil"+QString::number(li)+QString::number(li)+")*rotate(-90,mil"+QString::number(li)+QString::number(li)+")*debut"+QString::number(li)+QString::number(li)+";\n";
    texcode+="path arete"+QString::number(li)+QString::number(li)+"=debut"+QString::number(li)+QString::number(li)+"..milieu"+QString::number(li)+QString::number(li)+"..fin"+QString::number(li)+QString::number(li)+";\n";
//    texcode+="path arete"+QString::number(li)+QString::number(li)+"=point(sommet"+QString::number(li)+",-dir(135),t){-dir(135)}..{-dir(215)}point(sommet"+QString::number(li)+",-dir(45),t);\n"; 
    texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
    texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(li)+",.5),Relative(-dir(90)),Fill(1,white));\n";
    }
//  texcode+="draw(pic,arete"+QString::number(li)+QString::number(li)+",Arrow);\n";
//  texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(li)+",.5),2*Relative(dir(90)),Fill(1,white));\n";
  }
  for (int col = li+1; col < nbsommets; col++) 
    {
    poids=ligne.at(col);
    tligne=matrice.at(col);
    tpoids=tligne.at(li);
    extremite=sommetList.at(col);
    extremite_x=map_x.value(extremite);
    extremite_y=map_y.value(extremite);
    dir_x=extremite_x-origine_x;
    dir_y=extremite_y-origine_y;
    if (poids=="adjacent")
      {
      if (tpoids=="adjacent")
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";	
     texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";	
	      }
      else if (tpoids=="pasadjacent")
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";

	      }
      else
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";	
     texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";
      texcode+="label(pic,scale(.7)*\""+tpoids+"\",relpoint(arete"+QString::number(col)+QString::number(li)+",.5),Relative(dir(90+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))),Fill(1,white));\n";
	      }
      }
    else if (poids=="pasadjacent")
      {
      if (tpoids=="adjacent")
	      {
     texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";
/*	      texcode+="\\ncline{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}{"+QString::number(origine_x)+","+QString::number(origine_y)+"}\n"*/;
	      }
      else if (tpoids!="pasadjacent")
	      {
      texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";
      texcode+="label(pic,scale(.7)*\""+tpoids+"\",relpoint(arete"+QString::number(col)+QString::number(li)+",.5),Relative(dir(90+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))),Fill(1,white));\n";
// 	      texcode+="\\ncline{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}{"+QString::number(origine_x)+","+QString::number(origine_y)+"}^{"+tpoids+"}\n";
	      }
      }
    else
      {
      if (tpoids=="adjacent")
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";
     texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(col)+",.5),Relative(dir(90+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))),Fill(1,white));\n";
     texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";
 
// 	      texcode+="\\ncarc[arcangle=15]{"+QString::number(origine_x)+","+QString::number(origine_y)+"}{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}^{"+poids+"}\n";
// 	      texcode+="\\ncarc[arcangle=15]{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}{"+QString::number(origine_x)+","+QString::number(origine_y)+"}\n";
	      }
      else if (tpoids=="pasadjacent")
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";
     texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(col)+",.5),Relative(dir(90+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))),Fill(1,white));\n";
// 	      texcode+="\\ncline{"+QString::number(origine_x)+","+QString::number(origine_y)+"}{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}^{"+poids+"}\n";
	      }
      else
	      {
     texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+",Arrow);\n";
     texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(col)+",.5),Relative(dir(90+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))),Fill(1,white));\n";
     texcode+="path arete"+QString::number(col)+QString::number(li)+"=point(sommet"+QString::number(col)+",dir(degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t)";
      texcode+="{dir(25+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(li)+",dir(180+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(col)+QString::number(li)+",Arrow);\n";
      texcode+="label(pic,scale(.7)*\""+tpoids+"\",relpoint(arete"+QString::number(col)+QString::number(li)+",.5),Relative(dir(90+degrees(("+QString::number(-dir_x)+","+QString::number(-dir_y)+"),true))),Fill(1,white));\n";
// 	      texcode+="\\ncarc[arcangle=15]{"+QString::number(origine_x)+","+QString::number(origine_y)+"}{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}^{"+poids+"}\n";
// 	      texcode+="\\ncarc[arcangle=15]{"+QString::number(extremite_x)+","+QString::number(extremite_y)+"}{"+QString::number(origine_x)+","+QString::number(origine_y)+"}^{"+tpoids+"}\n";
	      }
      }
    }
  }  
}
else
{
for (int li = 0; li < nbsommets; li++)
  {
  ligne=matrice.at(li);
  origine=sommetList.at(li);
  origine_x=map_x.value(origine);
  origine_y=map_y.value(origine);
  for (int col = li+1; col < nbsommets; col++) 
    {
    poids=ligne.at(col);
    extremite=sommetList.at(col);
    extremite_x=map_x.value(extremite);
    extremite_y=map_y.value(extremite);
    dir_x=extremite_x-origine_x;
    dir_y=extremite_y-origine_y;

    if (poids=="adjacent")
      {
      texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+");\n";	    
      }
    else if (poids!="pasadjacent")
      {
      texcode+="path arete"+QString::number(li)+QString::number(col)+"=point(sommet"+QString::number(li)+",dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t)";
      texcode+="{dir(degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))}..";
      texcode+="point(sommet"+QString::number(col)+",dir(180+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true)),t);\n";
      texcode+="draw(pic,arete"+QString::number(li)+QString::number(col)+");\n";
      texcode+="label(pic,scale(.7)*\""+poids+"\",relpoint(arete"+QString::number(li)+QString::number(col)+",.5),Relative(dir(90+degrees(("+QString::number(dir_x)+","+QString::number(dir_y)+"),true))),Fill(1,white));\n";
      }
    }
  }
}
texcode+="});\n";
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void GrapheDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************************************
void GrapheDialog::ajouterSommet()
{
ActiverSommets(true);
sommetList.append("Nouveau");
aretesList.append("0");
QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetSommets);
nouveauItem->setText("Nouveau");
nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
ui.listWidgetSommets->setCurrentItem(nouveauItem);
ui.listWidgetSommets->setItemSelected(ui.listWidgetSommets->currentItem(), true);
ui.lineEditSommet->setFocus();
NettoyerCode();
}

void GrapheDialog::supprimerSommet()
{
int courant=numeroItem();
if (courant<0) return;
delete ui.listWidgetSommets->currentItem();
sommetList.removeAt(courant);
aretesList.removeAt(courant);
ui.listWidgetSommets->setCurrentItem(ui.listWidgetSommets->item(courant));
ui.listWidgetSommets->setItemSelected(ui.listWidgetSommets->item(courant),true);
actualiserItemInfo();
NettoyerCode();
}

void GrapheDialog::actualiserSommet()
{
int courant=numeroItem();
if (courant<0) return;
ui.listWidgetSommets->item(courant)->setText(ui.lineEditSommet->text());
sommetList[courant]=ui.lineEditSommet->text();
NettoyerCode();
}

void GrapheDialog::actualiserAretes()
{
int courant=numeroItem();
if (courant<0) return;
if (ui.lineEditAretes->text().isEmpty()) aretesList[courant]="0";
else aretesList[courant]=ui.lineEditAretes->text();
NettoyerCode();
}

void GrapheDialog::actualiserItemInfo()
{
int courant=numeroItem();
if (courant<0) 
	{
	ui.lineEditSommet->setText("");
	ui.lineEditAretes->setText("");
	ActiverSommets(false);
	}
else
	{
	ui.lineEditSommet->disconnect(SIGNAL(textChanged(const QString &)));
	ui.lineEditAretes->disconnect(SIGNAL(textChanged(const QString &)));

	ui.lineEditSommet->setText(sommetList[courant]);
	ui.lineEditAretes->setText(aretesList[courant]);

	connect(ui.lineEditSommet, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserSommet()));
	connect(ui.lineEditAretes, SIGNAL(textChanged(const QString &)),this,SLOT(actualiserAretes()));
	ActiverSommets(true);
	}
}

int GrapheDialog::numeroItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidgetSommets->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidgetSommets->setCurrentItem(items[0]);
	return ui.listWidgetSommets->row(items[0]);
	}
return -1;
}
//******************************************************
void GrapheDialog::NouveauGraphe()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Graphes : "),
					QString::fromUtf8("Le graphe courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverGraphe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Graphes : ")+nomFichier);
}

void GrapheDialog::SauverGraphe()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QStringList ligne;

QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Graphe");
doc.appendChild(root);

description=doc.createElement("Sommets");
contenu=doc.createTextNode(sommetList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Aretes");
contenu=doc.createTextNode(aretesList.join("#"));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Orientation");
if (ui.checkBoxOrientation->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Lignes");
contenu=doc.createTextNode(QString::number(nblignes));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Colonnes");
contenu=doc.createTextNode(QString::number(nbcolonnes));
root.appendChild(description);
description.appendChild(contenu);


for ( int li=0;li<nblignes;li++) 
	{
	ligne.clear();
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li,col);
		if (item) 
			{
			if (!item->text().isEmpty()) ligne.append(item->text());
			else ligne.append("vide");
			}
		else ligne.append("vide");
		}
	description=doc.createElement("LigneTableau");
	contenu=doc.createTextNode(ligne.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);
	}

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Graphe (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Graphe (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Graphes : ")+nomFichier);
}

void GrapheDialog::Ouvrir(QString nouveauFichier)
{
QTableWidgetItem *nouveau_item;
QStringList ligne;
int nblignes,nbcolonnes;
QString contenu;
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Graphe")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un graphe"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Sommets");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	sommetList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Aretes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	aretesList=node.toElement().text().split("#");
	}

nodelist = doc.elementsByTagName("Orientation");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxOrientation->setChecked(true);
	else ui.checkBoxOrientation->setChecked(false);
	}

nodelist = doc.elementsByTagName("Lignes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nblignes=node.toElement().text().toInt();
	}
nodelist = doc.elementsByTagName("Colonnes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nbcolonnes=node.toElement().text().toInt();
	}
ui.spinBoxLignes->setValue(nblignes);
ActualiserLignes(nblignes);
ui.spinBoxColonnes->setValue(nbcolonnes);
ActualiserColonnes(nbcolonnes);

nodelist = doc.elementsByTagName("LigneTableau");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	ligne=node.toElement().text().split("#");
	for (int col = 0; col < ligne.count(); col++)
		{
		contenu=ligne.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li, col,nouveau_item);
		}
	}


fichier.close();
ui.listWidgetSommets->clear();
for (int i = 0; i < sommetList.count(); i++)
	{
	if (!sommetList.at(i).isEmpty() && !aretesList.at(i).isEmpty())
		{
		QListWidgetItem *nouveauItem = new QListWidgetItem(ui.listWidgetSommets);
		nouveauItem->setText(sommetList.at(i));
		nouveauItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled);
		}
	else
		{
		sommetList.removeAt(i);
		aretesList.removeAt(i);
		}
	}
if (ui.listWidgetSommets->count()>0)
	{
	ui.listWidgetSommets->setCurrentItem(ui.listWidgetSommets->item(0));
	ui.listWidgetSommets->setItemSelected(ui.listWidgetSommets->currentItem(), true);
	}
actualiserItemInfo();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Graphes : ")+nomFichier);
}

void GrapheDialog::ChargerGraphe()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Graphes : "),
					QString::fromUtf8("Le graphe courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverGraphe();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Graphe (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************

//******************************
void GrapheDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Graphes ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void GrapheDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"graphe");
aideDialog->exec();
}
//*****************************
void GrapheDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void GrapheDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void GrapheDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void GrapheDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
out.setCodec(codec);
out << GenererCode();
fichier.close();
erreurProcessus=false;
asy_pdf_proc = new QProcess( this );
asy_pdf_proc->setWorkingDirectory(tempDir);
QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
#if defined(Q_OS_MAC)
env.insert("PATH", env.value("PATH") + ":/Library/TeX/texbin:/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin:/usr/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/texbin:/sw/bin:");
asy_pdf_proc->setProcessEnvironment(env);
#endif
#if defined(Q_OS_WIN32)
env.insert("PATH", env.value("PATH") + ";c:/texlive/2018/bin;c:/texlive/2017/bin;c:/texlive/2016/bin;c:/texlive/2015/bin;c:/texlive/2014/bin;c:/texlive/2013/bin;c:/texlive/2012/bin;c:/texlive/2011/bin;c:/texlive/2010/bin;c:/w32tex/bin;c:/Program Files/MiKTeX 3.0/miktex/bin;c:/Program Files (x86)/MiKTeX 3.0/miktex/bin;c:/Program Files/MiKTeX 2.9/miktex/bin;c:/Program Files (x86)/MiKTeX 2.9/miktex/bin;c:/Program Files/MiKTeX 2.8/miktex/bin;c:/Program Files (x86)/MiKTeX 2.8/miktex/bin");
#endif
qputenv("PATH", env.value("PATH").toLatin1());

connect(asy_pdf_proc, SIGNAL(finished(int)),this, SLOT(finAsyPdfProcess(int)));
asyMessage="";
connect(asy_pdf_proc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
asy_pdf_proc->start(asyCommande+" -f pdf -noView "+asyTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(this);
QProgressBar* progress = new QProgressBar();
progress->setFormat(" ");
progress->setMaximum(0);
progress->setMinimum(0);
progressDialog->setBar(progress);
progressDialog->setLabelText(QString::fromUtf8("Compilation en cours..."));
progressDialog->setCancelButton(0);
progressDialog->setRange(0,0);
progressDialog->setMinimumDuration(0);
progressDialog->show();
progressDialog->raise();
}

void GrapheDialog::readInfoOutput()
{
QByteArray result=asy_pdf_proc->readAllStandardError();
asyMessage=QString(result);
asyMessage=asyMessage.simplified();
}

void GrapheDialog::finAsyPdfProcess(int err)
{
if (progressDialog) progressDialog->cancel();
if (err) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation Asymptote")+"\n"+asyMessage);
	erreurProcessus=true;
	}
else 
	{
	emit asy_pdf_fini();
	}
}


void GrapheDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

