/***************************************************************************
 *   copyright       : (C) 2012-2014 by Pascal Brachet                     *
 *   http://www.xm1math.net/pdfadd/                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tabvardialog.h"
#include "aidedialog.h"
#include "logdialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QTableView>
#include <QTableWidgetItem>
#include <QColor>
#include <QFontMetrics>
#include <QUrl>
#include <QFontDatabase>
#include <QProgressBar>


TabvarDialog::TabvarDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouveauTableau()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerTableau()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverTableau()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(PdfLatexTemp()));
connect(this, SIGNAL(pdflatexfini()), this, SLOT(VoirTemp()));

connect(ui.tableWidget,SIGNAL(cellChanged(int,int)),this, SLOT(NettoyerCode()));
connect( ui.spinBoxLignes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserLignes(int)));
connect( ui.spinBoxColonnes, SIGNAL(valueChanged(int)),this, SLOT(ActualiserColonnes(int)));



tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
texTempFile=prefixFile+".tex";
pdfTempFile=prefixFile+".pdf";


highlighter = new LatexHighlighter(ui.textEditCode->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);

Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
show();
}

TabvarDialog::~TabvarDialog(){
if (pdflatexproc && pdflatexproc->state()==QProcess::Running) 
	{
	pdflatexproc->disconnect();
	pdflatexproc->kill(); 
	delete pdflatexproc ;
	}
if (progressDialog) delete progressDialog;
}

void TabvarDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void TabvarDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void TabvarDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void TabvarDialog::Init()
{
ui.textEditCode->clear();
ui.tableWidget->clear();
ui.spinBoxLignes->setValue(2);
ActualiserLignes(2);
ui.spinBoxColonnes->setValue(6);
ActualiserColonnes(6);
ui.tableWidget->setAcceptDrops(true);
ui.listWidgetMath->clear();
ui.listWidgetMath->setDragEnabled(true);
ui.listWidgetMath->addItems(QString("x;f(x);f^{\\,\\prime}(x);+;-;0;+\\infty;-\\infty").split(";"));
ui.listWidgetBarres->clear();
ui.listWidgetBarres->setDragEnabled(true);
ui.listWidgetBarres->addItems(QString("|;||;*").split(";"));
ui.listWidgetFleches->clear();
ui.listWidgetFleches->setDragEnabled(true);
ui.listWidgetFleches->addItems(QString("/;\\;_").split(";"));
}

void TabvarDialog::ActualiserLignes(int num)
{
QFontMetrics fontm=QFontMetrics(qApp->font());
int hauteur=fontm.lineSpacing()+4;
QStringList headerList;
for ( int i = 0; i <num; ++i )
	{
	headerList.append(QString::number(i+1)+" haut");
	headerList.append(QString::number(i+1)+" milieu");
	headerList.append(QString::number(i+1)+" bas");
	}
ui.tableWidget->setRowCount( num*3 );
ui.tableWidget->setVerticalHeaderLabels(headerList);
for ( int i = 0; i <num*3; ++i )
	{
	ui.tableWidget->setRowHeight(i,hauteur);
	}
ColorerTableau();
}

void TabvarDialog::ActualiserColonnes(int num)
{
QFontMetrics fontm=QFontMetrics(qApp->font());
int largeur=fontm.width( "wwwwwww" );
ui.tableWidget->setColumnCount( num );
for ( int i = 0; i <num; ++i )
	{
	ui.tableWidget->setColumnWidth(i,largeur);
	}
ColorerTableau();
}

void TabvarDialog::ColorerTableau()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item, *nouveau_item;
QColor couleurLigne;
QString contenu;
for ( int li=0;li<nblignes;li++) 
	{
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3, col,nouveau_item);
			}
		item=ui.tableWidget->item(li*3+1,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+1, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+1, col,nouveau_item);
			}
		item=ui.tableWidget->item(li*3+2,col);
		if (item)
			{
			contenu=item->text();
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText(contenu);
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+2, col,nouveau_item);
			}
		else
			{
			nouveau_item=new QTableWidgetItem();
			nouveau_item->setText("");
			nouveau_item->setBackgroundColor(couleurLigne);
			nouveau_item->setTextAlignment(Qt::AlignCenter);
			ui.tableWidget->setItem(li*3+2, col,nouveau_item);
			}
		}
	}
}

void TabvarDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

QString TabvarDialog::GenererCode()
{
QString haut,milieu,bas;
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QString texcode="\\tabvar{%\n";
for ( int li=0;li<nblignes;li++) 
	{
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item) haut=item->text();
		else haut="";
		item=ui.tableWidget->item(li*3+1,col);
		if (item) milieu=item->text();
		else milieu="";
		item=ui.tableWidget->item(li*3+2,col);
		if (item) bas=item->text();
		else bas="";
		if (haut=="|")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\txbt{ }";
				else texcode+="\\txbt{"+bas+"}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="*")
				{
				texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\txt{ }";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\traithaut";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\txt{"+milieu+"}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="||")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\dbt";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="*")
				{
				texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\doubletraithaut";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="/")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="*")
				{
				texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fmh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="\\")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="*")
				{
				texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fdh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="_")
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="*")
				{
				texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fhh";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else if (haut=="*")
			{
			if (milieu=="*")
				{
				if (bas=="*") texcode+="\\grise";
				else texcode+="\\tx{Erreur}";
				}
			else texcode+="\\tx{Erreur}";
			}
		else if (haut.isEmpty())
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\txht{ }";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\traitmilieu";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\doubletraitmilieu";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fm";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fd";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\fhm";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\traitbas";
				else if (bas=="||") texcode+="\\doubletraitbas";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\txb{"+bas+"}";
				}
			else
				{
				if (bas=="|") texcode+="\\xt{"+milieu+"}";
				else if (bas=="||") texcode+="\\xdbt{"+milieu+"}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{"+milieu+"}";
				else texcode+="\\tx{Erreur}";
				}
			}
		else
			{
			if (milieu=="|")
				{
				if (bas=="|") texcode+="\\txht{"+haut+"}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="||")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="/")
				{
				if (bas=="|") texcode+="\\trait";
				else if (bas=="||") texcode+="";
				else if ((bas=="/")) texcode+="\\fmb";
				else if (bas=="\\") texcode+="\\fdb";
				else if (bas=="_") texcode+="\\fhb";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="\\")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu=="_")
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			else if (milieu.isEmpty())
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\txh{"+haut+"}";
				else texcode+="\\tx{Erreur}";
				}
			else
				{
				if (bas=="|") texcode+="\\tx{Erreur}";
				else if (bas=="||") texcode+="\\tx{Erreur}";
				else if ((bas=="/")) texcode+="\\tx{Erreur}";
				else if (bas=="\\") texcode+="\\tx{Erreur}";
				else if (bas=="_") texcode+="\\tx{Erreur}";
				else if (bas=="*") texcode+="\\tx{Erreur}";
				else if (bas.isEmpty()) texcode+="\\tx{Erreur}";
				else texcode+="\\tx{Erreur}";
				}
			}
		if (col<nbcolonnes-1) texcode+="&";
		else texcode+="\\cr\n";
		}
	}
texcode+="}\n";
return texcode;
}

void TabvarDialog::AfficherCode()
{
QString texcode="%\\usepackage{fp}\n";
texcode+="%\\usepackage[pdftex]{pict2e}\n";
texcode+="%\\usepackage{picture}\n";
texcode+="%\\usepackage{color}\n";
texcode+="%\\input pdftabvar\n";
texcode+="\\[";
ui.textEditCode->clear();
texcode+=GenererCode()+"\\]\n";
ui.textEditCode->setPlainText(texcode);
}
//*********************************************
void TabvarDialog::PdfLatexTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile tabvar(":/documents/pdftabvar.txt");
tabvar.open(QIODevice::ReadOnly);
QString preambule;
QTextStream in(&tabvar);
while (!in.atEnd()) 
	{
	preambule+= in.readLine()+"\n";
	}
tabvar.close();
QFile fichier;
fichier.setFileName(tempDir+"/"+texTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
out << "\\documentclass[10pt,a4paper]{article}\n";
out << "\\usepackage[utf8]{inputenc}\n";
out << "\\usepackage[T1]{fontenc}\n";
out << "\\usepackage{amsmath}\n\\usepackage{amsfonts}\n\\usepackage{amssymb}\n";
out << "\\usepackage{fp}\n";
out << "\\usepackage[pdftex]{pict2e}\n";
out << "\\usepackage{picture}\n";
out << "\\usepackage{color}\n";
out << "\\usepackage[active,pdftex,tightpage]{preview}\n";
out << "\\pagestyle{empty}\n";
out << preambule;
out << "\\begin{document}\n";
out << "\\begin{preview}\n$\\,~";
out << GenererCode();
out << "\\,~$\\end{preview}\n";
out << "\\end{document}\n";
fichier.close();
erreurProcessus=false;
pdflatexproc = new QProcess( this );
pdflatexproc->setWorkingDirectory(tempDir);
QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
#if defined(Q_OS_MAC)
env.insert("PATH", env.value("PATH") + ":/Library/TeX/texbin:/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin:/usr/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/texbin:/sw/bin:");
pdflatexproc->setProcessEnvironment(env);
#endif
#if defined(Q_OS_WIN32)
env.insert("PATH", env.value("PATH") + ";c:/texlive/2018/bin;c:/texlive/2017/bin;c:/texlive/2016/bin;c:/texlive/2015/bin;c:/texlive/2014/bin;c:/texlive/2013/bin;c:/texlive/2012/bin;c:/texlive/2011/bin;c:/texlive/2010/bin;c:/w32tex/bin;c:/Program Files/MiKTeX 3.0/miktex/bin;c:/Program Files (x86)/MiKTeX 3.0/miktex/bin;c:/Program Files/MiKTeX 2.9/miktex/bin;c:/Program Files (x86)/MiKTeX 2.9/miktex/bin;c:/Program Files/MiKTeX 2.8/miktex/bin;c:/Program Files (x86)/MiKTeX 2.8/miktex/bin");
#endif
qputenv("PATH", env.value("PATH").toLatin1());

connect(pdflatexproc, SIGNAL(finished(int)),this, SLOT(finPdfLatexProcess(int)));
pdflatexproc->start(pdflatexCommande+" -interaction=nonstopmode "+texTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(this);
QProgressBar* progress = new QProgressBar();
progress->setFormat(" ");
progress->setMaximum(0);
progress->setMinimum(0);
progressDialog->setBar(progress);
progressDialog->setLabelText(QString::fromUtf8("Compilation en cours..."));
progressDialog->setCancelButton(0);
progressDialog->setRange(0,0);
progressDialog->setMinimumDuration(0);
progressDialog->show();
progressDialog->raise();
}

void TabvarDialog::finPdfLatexProcess(int err)
{
if (progressDialog) progressDialog->cancel();
if (err) 
	{
	LogDialog *logDlg = new LogDialog(this);
	logDlg->exec();
	//QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation LaTeX"));
	erreurProcessus=true;
	}
else 
	{
	emit pdflatexfini();
	}
}

void TabvarDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}

//******************************
void TabvarDialog::NouveauTableau()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Tableaux de variations : "),
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::SauverTableau()
{
int nblignes = ui.spinBoxLignes->value();
int nbcolonnes = ui.spinBoxColonnes->value();
QTableWidgetItem *item;
QStringList haut,milieu,bas;

QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Tableau");
doc.appendChild(root);

description=doc.createElement("Lignes");
contenu=doc.createTextNode(QString::number(nblignes));
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Colonnes");
contenu=doc.createTextNode(QString::number(nbcolonnes));
root.appendChild(description);
description.appendChild(contenu);


for ( int li=0;li<nblignes;li++) 
	{
	haut.clear();
	milieu.clear();
	bas.clear();
	for ( int col=0;col<nbcolonnes;col++)
		{
		item=ui.tableWidget->item(li*3,col);
		if (item) 
			{
			if (!item->text().isEmpty()) haut.append(item->text());
			else haut.append("vide");
			}
		else haut.append("vide");
		item=ui.tableWidget->item(li*3+1,col);
		if (item) 
			{
			if (!item->text().isEmpty()) milieu.append(item->text());
			else milieu.append("vide");
			}
		else milieu.append("vide");
		item=ui.tableWidget->item(li*3+2,col);
		if (item) 
			{
			if (!item->text().isEmpty()) bas.append(item->text());
			else bas.append("vide");
			}
		else bas.append("vide");
		}
	description=doc.createElement("Haut");
	contenu=doc.createTextNode(haut.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);

	description=doc.createElement("Milieu");
	contenu=doc.createTextNode(milieu.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);

	description=doc.createElement("Bas");
	contenu=doc.createTextNode(bas.join("#"));
	root.appendChild(description);
	description.appendChild(contenu);
	}

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Tableau (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Tableau (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;

QTableWidgetItem *nouveau_item;
QStringList haut,milieu,bas;
int nblignes,nbcolonnes;
QColor couleurLigne;
QString contenu;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Tableau")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un tableau"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("Lignes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nblignes=node.toElement().text().toInt();
	}
nodelist = doc.elementsByTagName("Colonnes");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	nbcolonnes=node.toElement().text().toInt();
	}
ui.spinBoxLignes->setValue(nblignes);
ActualiserLignes(nblignes);
ui.spinBoxColonnes->setValue(nbcolonnes);
ActualiserColonnes(nbcolonnes);

nodelist = doc.elementsByTagName("Haut");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	haut=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < haut.count(); col++)
		{
		contenu=haut.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3, col,nouveau_item);
		}
	}

nodelist = doc.elementsByTagName("Milieu");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	milieu=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < milieu.count(); col++)
		{
		contenu=milieu.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3+1, col,nouveau_item);
		}
	}

nodelist = doc.elementsByTagName("Bas");
for (int li = 0; li < nodelist.count(); li++)
	{
	node = nodelist.item(li);
	bas=node.toElement().text().split("#");
	if (li%2==0) couleurLigne=QColor("#ffffff");
	else couleurLigne=QColor("#eeeeee");
	for (int col = 0; col < bas.count(); col++)
		{
		contenu=bas.at(col);
		nouveau_item=new QTableWidgetItem();
		if (contenu!="vide") nouveau_item->setText(contenu);
		else nouveau_item->setText("");
		nouveau_item->setBackgroundColor(couleurLigne);
		nouveau_item->setTextAlignment(Qt::AlignCenter);
		ui.tableWidget->setItem(li*3+2, col,nouveau_item);
		}
	}

fichier.close();
AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Tableaux de variations : ")+nomFichier);
}

void TabvarDialog::ChargerTableau()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Tableaux de variations : ",
					QString::fromUtf8("Le tableau courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverTableau();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,QString::fromUtf8("Tableau (*.pag)"));
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}

//*******************************
void TabvarDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"tabvar");
aideDialog->exec();
}

//*****************************

void TabvarDialog::ExporterPdf()
{
this->disconnect(SIGNAL(pdflatexfini()));
connect(this, SIGNAL(pdflatexfini()), this, SLOT(ExporterPdfSuite()));
PdfLatexTemp();
}

void TabvarDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(pdflatexfini()));
connect(this, SIGNAL(pdflatexfini()), this, SLOT(VoirTemp()));
}

void TabvarDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

