/***************************************************************************
 *   copyright       : (C) 2012-2014 by Pascal Brachet                     *
 *   http://www.xm1math.net/pdfadd/                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "trigodialog.h"
#include "aidedialog.h"

#include <QDebug>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomProcessingInstruction>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QFileDialog>
#include <QFileInfo>
#include <QUrl>
#include <QFontDatabase>
#include <QProgressBar>
#include <QProcessEnvironment>



TrigoDialog::TrigoDialog(QWidget *parent)
    : QDialog( parent)
{
ui.setupUi(this);
#if defined(Q_OS_MAC)
setModal(true);
#endif

connect(ui.barreOutils->actionQuitter, SIGNAL(triggered()), this, SLOT(accept()));
connect(ui.barreOutils->actionGenerer, SIGNAL(triggered()), this, SLOT(AfficherCode()));
connect(ui.barreOutils->actionNouveau, SIGNAL(triggered()), this, SLOT(NouveauCercle()));
connect(ui.barreOutils->actionOuvrir, SIGNAL(triggered()), this, SLOT(ChargerCercle()));
connect(ui.barreOutils->actionSauver, SIGNAL(triggered()), this, SLOT(SauverCercle()));
connect(ui.barreOutils->actionAide, SIGNAL(triggered()), this, SLOT(Aide()));
connect(ui.barreOutils->actionExporterPdf, SIGNAL(triggered()), this, SLOT(ExporterPdf()));
connect(ui.barreOutils->actionApercu, SIGNAL(triggered()), this, SLOT(AsyPdfTemp()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));

connect(ui.checkBoxAnglesPar, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));
connect(ui.checkBoxValeursPar, SIGNAL(toggled(bool)),this, SLOT(NettoyerCode()));
connect(ui.textEditSupp, SIGNAL(textChanged()),this,SLOT(NettoyerCode()));

tempDir=QDir::tempPath();
prefixFile=QDir::homePath();
prefixFile="pdfadd_temp_"+prefixFile.section('/',-1);
prefixFile=QString(QUrl::toPercentEncoding(prefixFile));
prefixFile.remove("%");
asyTempFile=prefixFile+".asy";
pdfTempFile=prefixFile+".pdf";


highlighterSupp = new Highlighter(ui.textEditSupp->document());

QFont efont;
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Liberation Mono",Qt::CaseInsensitive)) deft="Liberation Mono";
else if (xf.contains("DejaVu Sans Mono",Qt::CaseInsensitive)) deft="DejaVu Sans Mono";
#if defined(Q_OS_MAC)
else if (xf.contains("Courier",Qt::CaseInsensitive)) deft="Courier";
#endif
#if defined(Q_OS_WIN32)
else if (xf.contains("Courier New",Qt::CaseInsensitive)) deft="Courier New";
#endif
else deft=qApp->font().family();  
efont.setFamily(deft);
efont.setPointSize(qApp->font().pointSize());
ui.textEditCode->setFont(efont);
ui.textEditCode->setWordWrapMode(QTextOption::NoWrap);
ui.textEditSupp->setFont(efont);
ui.textEditSupp->setWordWrapMode(QTextOption::NoWrap);
Init();

estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Cercle trigonométrique : ")+nomFichier);
show();

}

TrigoDialog::~TrigoDialog(){
if (asy_pdf_proc && asy_pdf_proc->state()==QProcess::Running) 
	{
	asy_pdf_proc->disconnect();
	asy_pdf_proc->kill(); 
	delete asy_pdf_proc ;
	}
if (progressDialog) delete progressDialog;
}

void TrigoDialog::closeEvent(QCloseEvent *e)
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Cercle trigonométrique : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCercle();
		break;
		case 1:
		break;
		case 2:
		default:
		e->ignore();
		return;
		break;
		}
	}
e->accept();
}

void TrigoDialog::accept()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Cercle trigonométrique : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCercle();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void TrigoDialog::reject()
{
if (pdfviewerWindow) pdfviewerWindow->close(); 
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,"Assistant Cercle trigonométrique : ",
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de quitter?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCercle();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::reject();
}

void TrigoDialog::Init()
{
ui.textEditCode->clear();
ui.checkBoxAnglesPar->setChecked(true);
ui.checkBoxValeursPar->setChecked(true);
ui.textEditSupp->clear();

}
//*********************************************


QString TrigoDialog::GenererCode()
{
QString texcode=QString::fromUtf8("// préambule asymptote\n")+preambuleAsy+"\n// code figure\n";
QFile cercleasy(":/documents/cercletrigoasy.txt");
cercleasy.open(QIODevice::ReadOnly);
QString preambule;
QTextStream in(&cercleasy);
while (!in.atEnd()) 
	{
	preambule+= in.readLine()+"\n";
	}
cercleasy.close();
texcode+=preambule;
if (ui.checkBoxAnglesPar->isChecked())
{
texcode+="ajouter_angle(pi/4,\"$\\frac{\\pi}{4}$\",true);\n";
texcode+="ajouter_angle(-pi/4,\"$-\\frac{\\pi}{4}$\",true);\n";
texcode+="ajouter_angle(3*pi/4,\"$\\frac{3\\pi}{4}$\",true);\n";
texcode+="ajouter_angle(-3*pi/4,\"$-\\frac{3\\pi}{4}$\",true);\n";
texcode+="ajouter_angle(pi/6,\"$\\frac{\\pi}{6}$\",true);\n";
texcode+="ajouter_angle(-pi/6,\"$-\\frac{\\pi}{6}$\",true);\n";
texcode+="ajouter_angle(pi/3,\"$\\frac{\\pi}{3}$\",true);\n";
texcode+="ajouter_angle(-pi/3,\"$-\\frac{\\pi}{3}$\",true);\n";
texcode+="ajouter_angle(2*pi/3,\"$\\frac{2\\pi}{3}$\",true);\n";
texcode+="ajouter_angle(-2*pi/3,\"$-\\frac{2\\pi}{3}$\",true);\n";
texcode+="ajouter_angle(5*pi/6,\"$\\frac{5\\pi}{6}$\",true);\n";
texcode+="ajouter_angle(-5*pi/6,\"$-\\frac{5\\pi}{6}$\",true);\n";  
}
if (ui.checkBoxValeursPar->isChecked())
{
texcode+="valeurs_particulieres();\n";
}
texcode+=QString::fromUtf8("// code supplémentaire\n")+ui.textEditSupp->toPlainText()+"\n"; 
texcode+="shipout(bbox(0.1cm,0.1cm,white));\n";
return texcode;
}

void TrigoDialog::AfficherCode()
{
ui.textEditCode->clear();
QString texcode=GenererCode()+"\n";
ui.textEditCode->setPlainText(texcode);
}

//*********************************************************************

//******************************************************
void TrigoDialog::NouveauCercle()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Cercle trigonométrique : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant d'en créer un nouveau?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCercle();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
Init();
estModifie=false;
nomFichier="sanstitre";
setWindowTitle(QString::fromUtf8("Assistant Cercle trigonométrique : ")+nomFichier);
}
void TrigoDialog::SauverCercle()
{
QDomDocument doc;
QDomElement description;
QDomText contenu;
QDomProcessingInstruction instr =  doc.createProcessingInstruction("xml","version=\"1.0\" encoding=\"UTF-8\"");
doc.appendChild(instr);
QDomElement root=doc.createElement("Cercle");
doc.appendChild(root);

description=doc.createElement("AnglesPar");
if (ui.checkBoxAnglesPar->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("ValeursPar");
if (ui.checkBoxValeursPar->isChecked()) contenu=doc.createTextNode("1");
else contenu=doc.createTextNode("0");
root.appendChild(description);
description.appendChild(contenu);

description=doc.createElement("Supplement");
contenu=doc.createTextNode(ui.textEditSupp->toPlainText());
root.appendChild(description);
description.appendChild(contenu);

QString nouveauFichier;
if (nomFichier=="sanstitre" ) 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Cercle (*.pag)"));
	}
else 
	{
	nouveauFichier= QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),nomFichier,QString::fromUtf8("Cercle (*.pag)"));
	}
//nouveauFichier=nomFichier;
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible d'enregistrer le fichier"));
	return;
	}
nomFichier=nouveauFichier;
QTextStream out (&fichier);
doc.save(out,4);
fichier.close();
QFileInfo fi(nomFichier);
dernierRepertoire=fi.absolutePath();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Cercle trigonométrique : ")+nomFichier);
}

void TrigoDialog::Ouvrir(QString nouveauFichier)
{
QDomDocument doc;
QDomNodeList nodelist;
QDomNode node;
QFileInfo fi(nouveauFichier);
if (!fi.exists()) return;
dernierRepertoire=fi.absolutePath();
QFile fichier(nouveauFichier);
if (!fichier.open(QIODevice::ReadOnly)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	return;
	}
if (!doc.setContent(&fichier)) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Impossible de lire le fichier"));
	fichier.close();
	return;
	}

QDomElement root = doc.documentElement();
if (root.tagName() != "Cercle")
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Ce fichier ne correspond pas à un cercle trigonométrique"));
	fichier.close();
	return;
	}
nomFichier=nouveauFichier;
Init();

nodelist = doc.elementsByTagName("AnglesPar");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxAnglesPar->setChecked(true);
	else ui.checkBoxAnglesPar->setChecked(false);
	}

nodelist = doc.elementsByTagName("ValeursPar");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	if (node.toElement().text()=="1") ui.checkBoxValeursPar->setChecked(true);
	else ui.checkBoxValeursPar->setChecked(false);
	}
	
nodelist = doc.elementsByTagName("Supplement");
for (int i = 0; i < nodelist.count(); i++)
	{
	node = nodelist.item(i);
	ui.textEditSupp->setPlainText(node.toElement().text());
	}

AfficherCode();
estModifie=false;
setWindowTitle(QString::fromUtf8("Assistant Cercle trigonométrique : ")+nomFichier);
}

void TrigoDialog::ChargerCercle()
{
if (estModifie) 
	{
	switch(  QMessageBox::warning(this,QString::fromUtf8("Assistant Cercle trigonométrique : "),
					QString::fromUtf8("Le graphique courant a été modifié.\nVoulez-vous l'enregistrer avant de charger un autre graphe?"),
					QString::fromUtf8("Enregistrer"), QString::fromUtf8("Ne pas enregistrer"), QString::fromUtf8("Abandon"),
					0,
					2 ) )
		{
		case 0:
		SauverCercle();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString nouveauFichier = QFileDialog::getOpenFileName(this,QString::fromUtf8("Ouvrir Fichier"),dernierRepertoire,"Cercle (*.pag)");
if (nouveauFichier.isEmpty()) return;
Ouvrir(nouveauFichier);
}
//*********************************************
void TrigoDialog::NettoyerCode()
{
ui.textEditCode->clear();
estModifie=true;
setWindowTitle(QString::fromUtf8("Assistant Cercle trigonométrique ")+QString::fromUtf8("[modifié]")+" : "+nomFichier);
}

//*******************************
void TrigoDialog::Aide()
{
AideDialog *aideDialog = new AideDialog(this,"cercle");
aideDialog->exec();
}
//*****************************
void TrigoDialog::ExporterPdf()
{
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(ExporterPdfSuite()));
AsyPdfTemp();
}

void TrigoDialog::ExporterPdfSuite()
{
if (!erreurProcessus)
	{
	QString pdfFichier = QFileDialog::getSaveFileName(this,QString::fromUtf8("Enregistrer sous"),dernierRepertoire,QString::fromUtf8("Fichier Pdf (*.pdf)"));
	if (!pdfFichier.isEmpty())
		{
		QFileInfo fi(pdfFichier);
		dernierRepertoire=fi.absolutePath();
		CopierFichier(tempDir+"/"+pdfTempFile,pdfFichier);
		}
	}
this->disconnect(SIGNAL(asy_pdf_fini()));
connect(this, SIGNAL(asy_pdf_fini()), this, SLOT(VoirTemp()));
}

void TrigoDialog::CopierFichier(QString origine,QString destination)
{
if (destination.isEmpty() || origine.isEmpty()) return;
QFileInfo fi_or(origine);
if (!fi_or.exists()) return;
QFile fichier_or(origine);
QFileInfo fi_dest(destination);
if (fi_dest.exists())
	{
	QFile fichier_dest(destination);
	fichier_dest.remove();
	fichier_or.copy(destination);
	}
else
	{
	fichier_or.copy(destination);
	}
}

void TrigoDialog::AsyPdfTemp()
{
AfficherCode();
if (ui.textEditCode->toPlainText().contains(QRegExp("Erreur|erreur"))) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Erreurs dans le code généré.\nLa compilation n'est pas lancée."));
	return;
	}
QFile fichier;
fichier.setFileName(tempDir+"/"+asyTempFile);
fichier.open(QIODevice::WriteOnly);
QTextStream out (&fichier);
QTextCodec *codec = QTextCodec::codecForName("UTF-8");
out.setCodec(codec);
out << GenererCode();
fichier.close();
erreurProcessus=false;
asy_pdf_proc = new QProcess( this );
asy_pdf_proc->setWorkingDirectory(tempDir);

QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
#if defined(Q_OS_MAC)
env.insert("PATH", env.value("PATH") + ":/Library/TeX/texbin:/Library/TeX/Distributions/.DefaultTeX/Contents/Programs/texbin:/usr/bin:/usr/sbin:/sbin:/usr/local/bin:/usr/texbin:/sw/bin:");
asy_pdf_proc->setProcessEnvironment(env);
#endif
#if defined(Q_OS_WIN32)
env.insert("PATH", env.value("PATH") + ";c:/texlive/2018/bin;c:/texlive/2017/bin;c:/texlive/2016/bin;c:/texlive/2015/bin;c:/texlive/2014/bin;c:/texlive/2013/bin;c:/texlive/2012/bin;c:/texlive/2011/bin;c:/texlive/2010/bin;c:/w32tex/bin;c:/Program Files/MiKTeX 3.0/miktex/bin;c:/Program Files (x86)/MiKTeX 3.0/miktex/bin;c:/Program Files/MiKTeX 2.9/miktex/bin;c:/Program Files (x86)/MiKTeX 2.9/miktex/bin;c:/Program Files/MiKTeX 2.8/miktex/bin;c:/Program Files (x86)/MiKTeX 2.8/miktex/bin");
#endif

qputenv("PATH", env.value("PATH").toLatin1());

connect(asy_pdf_proc, SIGNAL(finished(int)),this, SLOT(finAsyPdfProcess(int)));
asyMessage="";
connect(asy_pdf_proc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoOutput()));
asy_pdf_proc->start(asyCommande+" -f pdf -noView "+asyTempFile);
if (!progressDialog) progressDialog = new QProgressDialog(this);
QProgressBar* progress = new QProgressBar();
progress->setFormat(" ");
progress->setMaximum(0);
progress->setMinimum(0);
progressDialog->setBar(progress);
progressDialog->setLabelText(QString::fromUtf8("Compilation en cours..."));
progressDialog->setCancelButton(0);
progressDialog->setRange(0,0);
progressDialog->setMinimumDuration(0);
progressDialog->show();
progressDialog->raise();
}

void TrigoDialog::readInfoOutput()
{
QByteArray result=asy_pdf_proc->readAllStandardError();
asyMessage=QString(result);
asyMessage=asyMessage.simplified();
}

void TrigoDialog::finAsyPdfProcess(int err)
{
if (progressDialog) progressDialog->cancel();
if (err) 
	{
	QMessageBox::warning( this,QString::fromUtf8("Erreur"),QString::fromUtf8("Echec de la compilation Asymptote")+"\n"+asyMessage);
	erreurProcessus=true;
	}
else 
	{
	emit asy_pdf_fini();
	}
}


void TrigoDialog::VoirTemp()
{
if (erreurProcessus) return;
QString fileImage=tempDir+"/"+pdfTempFile;
QFileInfo fi(fileImage);
if (fi.exists()) 
	{
	if (pdfviewerWindow)
	  {
	  pdfviewerWindow->openFile(fileImage,lecteurpdfCommande);
	  }
	else
	  {
	  pdfviewerWindow=new PdfViewer(fileImage,lecteurpdfCommande,this);
	  }  
	  pdfviewerWindow->raise();
	  pdfviewerWindow->show();
	}
}
